@@ -590,7 +590,7 @@
      (if (feature-supported? "defer") '()
          `(,(wfc-error
              o
-             (format #f "use of `blocking' with language ~s is not supported"
+             (format #f "use of `defer' with language ~s is not supported"
                      (%language)))))
      (if (not arguments) '()
          (append-map wfc-defer-argument arguments))
@@ -1059,9 +1059,9 @@
     (if (not duplicate?) '()
         (let ((name (if (ast:parent (.parent o) <model>) (ast:name o)
                         (ast:dotted-name o))))
-         `(,(wfc-error o (format #f "identifier `~a' defined before" name))
-           ,(wfc-info previous (format #f "previous `~a' definition here"
-                                       name)))))))
+          `(,(wfc-error o (format #f "identifier `~a' defined before" name))
+            ,(wfc-info previous (format #f "previous `~a' definition here"
+                                        name)))))))
 
 (define-method (assign (o <ast>))
   (or (as (wfc (.expression o)) <pair>)
