@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
 ;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024 Janneke Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2020, 2021, 2022, 2023, 2024 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2020, 2021, 2022, 2023, 2024, 2025 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2016, 2017, 2018, 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2018 Filip Toman <filip.toman@verum.com>
 ;;; Copyright © 2023 Paul Hoogendijk <paul@dezyne.org>
@@ -66,6 +66,8 @@
             code:add-calling-context-formal
             code:annotate-shells
             code:argument*
+            code:blocking-requires-in-void-returns
+            code:blocking-return-values
             code:blocking?
             code:capture-local
             code:capture-name
@@ -466,18 +468,13 @@
 ;;;
 ;;; Constraint.
 ;;;
-(define-method (code:capture-name (o <variable>) (d <defer>))
-  (string-append "dzn"
-                 "_"
-                 "capture"
-                 "_"
-                 (.name o)
-                 "_"
-                 (number->string (.offset (.location d)))))
+(define-method (code:capture-name (o <variable>))
+  (string-join (list "dzn" "capture" (.name o)) "_"))
 
 (define-method (code:defer-condition (o <defer>))
   (not (or (and=> (.arguments o)(compose null? .elements))
-           (null? (ast:variable* (ast:parent o <component>))))))
+           (null? (filter (compose not (is? <extern>) ast:type)
+                          (ast:variable* (ast:parent o <component>)))))))
 
 (define-method (code:capture-local (o <defer>))
   (let* ((references (tree-collect (disjoin(is? <assign>)
@@ -499,6 +496,7 @@
 ;;;
 ;;; Shared state.
 ;;;
+(define flush-regexp (make-regexp "'flush"))
 (define trigger-regexp (make-regexp "'in\\(([^)]*\\))\\)"))
 (define action-regexp (make-regexp "'out\\(([^)]*\\))\\)"))
 (define reply-regexp (make-regexp "'reply\\(([^)]*\\))\\)"))
@@ -529,6 +527,9 @@
 
 (define-method (code:shared (o <event>))
   "Return a list of transitions for event O from the interface LTS"
+  (define (flush? o)
+    (and=> (regexp-exec flush-regexp o)
+           (cute match:substring <> 0)))
   (define (trigger? o)
     (and=> (regexp-exec trigger-regexp o)
            (cute match:substring <> 1)))
@@ -554,7 +555,9 @@
       (_
        o)))
   (define (event->prefix event) ;; XXX vouw in trigger?, action?, reply?
-    (or (and=> (regexp-exec in-regexp event)
+    (or (and (regexp-exec flush-regexp event)
+             "<flush>")
+        (and=> (regexp-exec in-regexp event)
                (cute match:substring <> 1))
         (and=> (regexp-exec out-regexp event)
                (cute match:substring <> 1))
@@ -568,7 +571,7 @@
         ;; HACK for debugging
         (pke "EVENT NOT FOUND" event)))
   (define (edge->transition edge)
-    (let* ((label ((disjoin trigger? action? reply?) (edge-label edge)))
+    (let* ((label ((disjoin trigger? action? reply? flush?) (edge-label edge)))
            (label (event->prefix label)))
       (set-field edge (edge-label) label)))
   (define (to=from edge)
@@ -583,7 +586,7 @@
                                          (compose (cute equal? <>
                                                         "declarative_illegal")
                                                   edge-label)
-                                                 to=from) edges)))
+                                         to=from) edges)))
          (edges (filter (compose not
                                  (cute member <> illegal-node-ids)
                                  edge-to) edges))
@@ -627,13 +630,14 @@
                (make <shared-state> #:state from #:assign compound)))))
     (let* ((debugity (dzn:debugity))
            (lts (code:shared-lts o))
+           (initial (initial lts))
            (nodes (vector->list lts))
            (edges (append-map node-edges nodes))
            (assign (filter-map edge->assign edges)))
       (when (> debugity 2)
         (display "code:\n" (current-error-port))
         (display assign (current-error-port)))
-      assign)))
+      (values assign initial))))
 
 (define-method (code:shared-value (o <shared-value>))
   (string-replace-substring (.value o) "'" (%type-infix)))
@@ -717,25 +721,34 @@
 ;;;
 ;;; Main.
 ;;;
+(define (trigger->port-pairs trigger)
+  (map (cute make
+             <port-pair>
+             #:port (.port.name trigger)
+             #:other <>)
+       (map ->sexp (ast:return-values trigger))))
+
 (define-method (code:return-values (o <component-model>))
-  (define (trigger->port-pairs trigger)
-    (map (cute make
-               <port-pair>
-               #:port (.port.name trigger)
-               #:other <>)
-         (map ->sexp (ast:return-values trigger))))
   (let* ((triggers (filter (compose not (is? <void>) ast:type)
                            (ast:requires-in-triggers o)))
          (pairs (append-map trigger->port-pairs triggers)))
-    (delete-duplicates pairs (lambda (a b)
-                               (and (ast:eq? (.port a) (.port b))
-                                    (equal? (.other a) (.other b)))))))
+    (delete-duplicates pairs ast:equal?)))
+
+(define-method (code:blocking-return-values (o <component-model>))
+  (let* ((triggers (filter (conjoin (compose .blocking? .port)
+                                    (compose not (is? <void>) ast:type))
+                           (ast:requires-in-triggers o)))
+         (pairs (append-map trigger->port-pairs triggers)))
+    (delete-duplicates pairs ast:equal?)))
 
 (define-method (code:requires-in-void-returns (o <component-model>))
   (let ((triggers (ast:requires-in-void-triggers o)))
-    (delete-duplicates triggers
-                       (lambda (a b)
-                         (ast:eq? (.port a) (.port b))))))
+    (delete-duplicates triggers ast:port-eq?)))
+
+(define-method (code:blocking-requires-in-void-returns (o <component-model>))
+  (let ((triggers (filter (compose .blocking? .port)
+                          (ast:requires-in-void-triggers o))))
+    (delete-duplicates triggers ast:port-eq?)))
 
 
 ;;;
