@@ -203,7 +203,7 @@ and \"POST\" 'post in GRAMMAR."
 
 (define-method (print-ast (o <namespace>) port)
   (display "namespace " port)
-  (display (ast:dotted-name o) port)
+  (display (ast:name o) port)
   (print-newline port)
   (print-brace-open port)
   (print-ast-join (dzn:top* o) port "\n")
@@ -225,7 +225,7 @@ and \"POST\" 'post in GRAMMAR."
 
 (define-method (print-ast (o <enum>) port)
   (simple-format port "enum ")
-  (print-type o port)
+  (display (ast:name o) port)
   (display " {" port)
   (print-ast-join (ast:field* o) port ", ")
   (display "};\n" port))
@@ -460,9 +460,12 @@ and \"POST\" 'post in GRAMMAR."
       (display ");\n" port)))))
 
 (define-method (print-ast (o <return>) port)
-  (display "return " port)
-  (print-ast (.expression o) port)
-  (display ";\n" port))
+  (let ((expression (.expression o)))
+    (display "return" port)
+    (when (ast:typed? expression)
+      (display " ")
+      (print-ast expression port))
+    (display ";\n" port)))
 
 (define-method (print-ast (o <if>) port)
   (let ((else (.else o)))
@@ -495,8 +498,7 @@ and \"POST\" 'post in GRAMMAR."
   (display (.name o) port))
 
 (define-method (print-ast (o <shared-var>) port)
-  (let ((lst (ast:full-name o)))
-    (display (string-join lst ".") port)))
+  (simple-format port "~a.~a" (.name (.port o)) (.name o)))
 
 (define-method (print-ast (o <binary>) port)
   (print-ast (.left o) port)
@@ -518,10 +520,7 @@ and \"POST\" 'post in GRAMMAR."
   (display (.field o) port))
 
 (define-method (print-ast (o <shared-field-test>) port)
-  (let ((lst (ast:full-name o)))
-    (display (string-join lst ".") port))
-  (display "." port)
-  (display (.field o) port))
+  (simple-format port "~a.~a.~a" (.name (.port o)) (.name o) (.field o)))
 
 (define-method (print-ast (o <enum-literal>) port)
   (print-type (.type o) port)
