@@ -573,17 +573,21 @@
 
 (define (makreel:add-state-placeholder o)
   (match o
-    ((and (? (is? <compound>))
-          (? (cute ast:parent <> <interface>))
-          (? (cute ast:parent <> <on>)))
+    ((? (is? <component>)) o)
+    ((? (is? <compound>))
      (clone o #:elements (makreel:add-state-placeholder (ast:statement* o))))
+    (((and (? (is? <if>)) s) rest ...)
+     (cons (clone s #:expression (.expression s)
+                  #:then (makreel:add-state-placeholder (.then s))
+                  #:else (and=> (.else s) makreel:add-state-placeholder))
+           (makreel:add-state-placeholder rest)))
     (((and (? (is? <action>)) statement) rest ...)
-     (let* ((trigger (car (ast:trigger* (ast:parent statement <on>)))))
-       (cons* statement
-              (make <state>)
-              (makreel:add-state-placeholder rest))))
+     (cons* statement
+            (make <state>)
+            (makreel:add-state-placeholder rest)))
     (((and (? (is? <statement>)) statement) rest ...)
-     (cons statement (makreel:add-state-placeholder rest)))
+     (cons (makreel:add-state-placeholder statement)
+           (makreel:add-state-placeholder rest)))
     ((? (is? <ast>))
      (tree-map makreel:add-state-placeholder o))
     (_ o)))
@@ -599,12 +603,12 @@ transformations."
                   add-function-return
                   extract-call
                   makreel:add-state-placeholder
+                  (if (%no-unreachable?) identity tag-imperative-blocks)
                   normalize:state+illegals
                   remove-otherwise
                   makreel:tick-names
                   add-explicit-temporaries
                   add-defer-end
-                  (if (%no-unreachable?) identity tag-imperative-blocks)
                   purge-data
                   ) ast)))
       (when (> (dzn:debugity) 1)
