@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2023, 2024 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2022, 2023, 2024 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2023, 2024, 2025 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -275,14 +275,14 @@ std::basic_ostream<Char, Traits> &")
         (sm:assign* var (ast->code expression)))))
 
 (define-method (ast->code (o <defer>))
-  (define (variable->defer-variable o)
+  (define (variable->defer-variable v)
     (sm:variable
-     (type (code:type-name (ast:type o)))
-     (name (code:capture-name o))
-     (expression (code:member-name o))))
-  (define (variable->equality o)
-    (sm:equal* (c++:ast->expression o)
-               (code:capture-name o)))
+     (type (code:type-name (ast:type v)))
+     (name (c++:capture-name v o))
+     (expression (code:member-name v))))
+  (define (variable->equality v)
+    (sm:equal* (c++:ast->expression v)
+               (c++:capture-name v o)))
   (let* ((variables (ast:defer-variable* o))
          (locals (code:capture-local o))
          (equality (code:defer-equality* o))
@@ -301,8 +301,9 @@ std::basic_ostream<Char, Traits> &")
                         (list "this"
                               (sm:function
                                (captures
-                                (cons* "this"
-                                       (map code:capture-name variables)))
+                                `("this"
+                                  ,@(map (cute c++:capture-name <> o)
+                                         variables)))
                                (statement
                                 (sm:compound*
                                  (sm:return* condition))))
@@ -328,7 +329,7 @@ std::basic_ostream<Char, Traits> &")
                  `(,(sm:assign*
                      (sm:member* (string-append "*" (%member-prefix))
                                  (code:reply-var type))
-                                (c++:ast->expression (.expression o)))))
+                     (c++:ast->expression (.expression o)))))
            ,(sm:if* out-binding (sm:call (name out-binding)))
            ,(sm:assign* out-binding "nullptr")
            ,@(if (not (code:port-release? o)) '()
@@ -444,7 +445,8 @@ std::basic_ostream<Char, Traits> &")
               (sm:statement*
                (format #f "throw dzn::binding_error (this->dzn_meta, ~s)"
                        event-name)))))
-  (let* ((public-enums (code:public-enum* o))
+  (let* ((shared initial (code:shared-state o))
+         (public-enums (code:public-enum* o))
          (enums (append public-enums (code:enum* o)))
          (interface
           (sm:struct
@@ -465,7 +467,7 @@ std::basic_ostream<Char, Traits> &")
                              (name "out"))
                ,(sm:variable (type "bool") (name "dzn_share_p") (expression "true"))
                ,(sm:variable (type "char const*") (name "dzn_label") (expression "\"\""))
-               ,(sm:variable (type "int") (name "dzn_state") (expression ""))
+               ,(sm:variable (type "int") (name "dzn_state") (expression initial))
                ,@(map code:member->variable (ast:member* o))))))
          (interface& (string-append
                       (string-join (cons "" (ast:full-name o)) "::")
@@ -569,11 +571,18 @@ std::basic_ostream<Char, Traits> &")
                                         (name "require"))))
               (statement
                (sm:compound*
-                `(,@(map (lambda (o)
-                           (sm:assign*
-                            (string-append "provide.out." (.name o))
-                            (string-append "require.out." (.name o))))
-                         (ast:out-event* o))
+                `(,@(append-map
+                     (lambda (o)
+                       (let ((provide (string-append "provide.out." (.name o)))
+                             (require (string-append "require.out." (.name o))))
+                         `(,(sm:assign*
+                             (string-append provide ".other_port_update")
+                             (string-append require ".port_update"))
+                           ,(sm:assign*
+                             (string-append require ".other_port_update")
+                             (string-append provide ".port_update"))
+                           ,(sm:assign* provide require))))
+                     (ast:out-event* o))
                   ,@(map (lambda (o)
                            (sm:assign*
                             (string-append "require.in." (.name o))
@@ -674,8 +683,7 @@ std::basic_ostream<Char, Traits> &")
                                            (name "locator"))))
                            (statement
                             (sm:compound*))))
-                       (let* ((shared (code:shared-state o))
-                              (events (ast:event* o))
+                       (let* ((events (ast:event* o))
                               (transitions (append-map event->transitions
                                                        events))
                               (transitions (sort transitions transition<))
@@ -759,14 +767,14 @@ std::basic_ostream<Char, Traits> &")
      (type (code:type-name (.type port)))
      (name (.name port))
      (expression (simple-format
-                  #f "{{~s,&~a,this,&dzn_meta},{~s,0,0,0}},this" name name name))))
+                  #f "{{~s,&~a,this,&dzn_meta},{~s,&~a,0,0}},this" name name name name))))
   (define (requires->member port)
     (sm:variable
      (type (code:type-name (.type port)))
      (name (.name port))
      (expression (simple-format
                   #f
-                  "{{~s,0,0,0},{~s,&~a,this,&dzn_meta}},this" name name name))))
+                  "{{~s,&~a,0,0},{~s,&~a,this,&dzn_meta}},this" name name name name))))
   (define (requires->external port)
     (sm:assign*
      (simple-format
@@ -826,11 +834,11 @@ std::basic_ostream<Char, Traits> &")
           `(,@(map port->injected-require-override
                    (ast:injected-port* o))
             ,@(if (not (ast:provides? port)) '()
-               `(,(sm:assign*
-                   (sm:member* (%member-prefix) (code:out-binding port))
-                   (sm:member* (string-append "&" (%member-prefix))
-                               (string-append (code:event-name trigger)
-                                              ".dzn_out_binding")))))
+                  `(,(sm:assign*
+                      (sm:member* (%member-prefix) (code:out-binding port))
+                      (sm:member* (string-append "&" (%member-prefix))
+                                  (string-append (code:event-name trigger)
+                                                 ".dzn_out_binding")))))
             ,@(if (not typed?) `(,call-slot)
                   `(,(sm:assign*
                       (sm:member* (%member-prefix) (code:reply-var return-type))
@@ -955,13 +963,14 @@ std::basic_ostream<Char, Traits> &")
                                            (name "locator"))))
                  (statement
                   (sm:compound*
-                   `(,@(map requires->external (filter ast:external? (ast:requires-port* o)))
+                   `(,@(map requires->external (filter ast:external?
+                                                       (ast:requires-port* o)))
                      ,@(if (is-a? o <foreign>) '()
-                           `(,(requires->assignment (ast:requires-port* o))
+                           `(,(sm:assign* "dzn_runtime.native (this)"
+                                          "true")
+                             ,(requires->assignment (ast:requires-port* o))
                              ,@(append-map injected->assignments
-                                           (ast:injected-port* o))
-                             ,(sm:assign* (sm:member* "dzn_runtime.performs_flush (this)")
-                                          "true")))
+                                           (ast:injected-port* o))))
                      ,@(map trigger->event-slot (ast:in-triggers o))))))
                ,@(if (not (is-a? o <foreign>)) '()
                      `(,(sm:destructor (struct component)
@@ -987,10 +996,12 @@ std::basic_ostream<Char, Traits> &")
     (c++:include-guard
      o
      `(,@(map c++:file-name->include interface-includes)
-       ,@(component-model->statements o)))))
+       ,@(component-model->statements o)
+       ,@(append-map c++:enum->statements (code:enum* o))))))
 
 (define-method (model->statements (o <component>))
-  (component-model->statements o))
+  `(,@(component-model->statements o)
+    ,@(append-map c++:enum->statements (code:enum* o))))
 
 
 ;;;
@@ -1174,14 +1185,16 @@ std::basic_ostream<Char, Traits> &")
          (type (string-join (cons "" (ast:full-name (.type port))) "::"))
          (name (.name port))
          (expression
-          "{}"))))
+          (simple-format
+           #f "{{~s,&~a,this,&dzn_meta},{~s,0,0,0}}" name name name)))))
     (define (requires->member port)
       (let ((other-end (ast:other-end-point port)))
         (sm:variable
          (type (string-join (cons "" (ast:full-name (.type port))) "::"))
          (name (.name port))
          (expression
-          "{}"))))
+          (simple-format
+           #f "{{~s,0,0,0},{~s,&~a,this,&dzn_meta}}" name name name)))))
     (define (injected-instance? instance)
       (member instance injected-instances ast:eq?))
     (define (instance->member instance)
@@ -1392,23 +1405,27 @@ std::basic_ostream<Char, Traits> &")
            (system-port (string-append "c.system." port))
            (meta (string-append system-port ".dzn_meta"))
            (port-event (string-append port "." event))
+           (blocking? (.blocking? (.port trigger)))
            (function
             (sm:function
              (captures '("&"))
              (formals formals)
              (statement
               (sm:compound*
+               (sm:call (name "c.perform"))
                (sm:call (name "c.match")
                         (arguments
                          (list (simple-format #f "~s" port-event))))
-               (sm:call (name "dzn::port_block")
-                        (arguments
-                         (list "c.dzn_locator" "nullptr"
-                               (simple-format #f "&c.system.~a" port))))
+               (if (not blocking?) (sm:call (name "c.sync_trigger"))
+                   (sm:call (name "dzn::port_block")
+                            (arguments
+                             (list "c.dzn_locator" "nullptr"
+                                   (simple-format #f "&c.system.~a" port)))))
                (sm:variable (type "std::string")
                             (name "tmp")
                             (expression
-                             (sm:call (name "c.trail_expect"))))
+                             (sm:call (name "c.trail.front"))))
+               (sm:call (name "c.trail.pop"))
                (sm:variable (type "size_t")
                             (name "pos")
                             (expression "tmp.rfind ('.')+1"))
@@ -1439,6 +1456,7 @@ std::basic_ostream<Char, Traits> &")
             (formals formals)
             (statement
              (sm:compound*
+              (sm:call (name "c.perform"))
               (sm:call (name "c.match")
                        (arguments
                         (list (simple-format #f "~s" port-event))))))))))))
@@ -1472,14 +1490,24 @@ std::basic_ostream<Char, Traits> &")
   (define (requires->init port)
     (let* ((port-name (.name port))
            (system-port (simple-format #f "c.system.~a" port-name))
-           (meta (simple-format #f "~a.dzn_meta.provide" system-port)))
+           (meta (simple-format #f "~a.dzn_meta.provide" system-port))
+           (requires (simple-format #f "~a.dzn_meta.require.component"
+                                    system-port)))
       (list
        (sm:assign (variable (simple-format #f "~a.component" meta))
-                  (expression "&c"))
+                  (expression "nullptr"))
        (sm:assign (variable (simple-format #f "~a.meta" meta))
                   (expression "&c.dzn_meta"))
        (sm:assign (variable (simple-format #f "~a.name" meta))
-                  (expression (simple-format #f "~s" port-name))))))
+                  (expression (simple-format #f "~s" port-name)))
+       (sm:assign (variable
+                   (sm:call (name "c.dzn_runtime.performs_flush")
+                            (arguments (list requires))))
+                  (expression "flush")))))
+  (define (formal->variable formal)
+    (sm:variable (type (code:type-name formal))
+                 (name (code:number-argument formal))
+                 (expression (.name formal))))
   (define (void-provides-in->init trigger)
     (let* ((port (.port.name trigger))
            (event (.event.name trigger))
@@ -1491,10 +1519,6 @@ std::basic_ostream<Char, Traits> &")
            (port-event-string (simple-format #f "~s" port-event))
            (port-return (simple-format #f "~a.return" port))
            (port-sm:return-string (simple-format #f "~s" port-return)))
-      (define (formal->variable formal)
-        (sm:variable (type "int")
-                     (name (code:number-argument formal))
-                     (expression (.name formal))))
       (sm:generalized-initializer-list*
        port-event-string
        (sm:function
@@ -1506,35 +1530,41 @@ std::basic_ostream<Char, Traits> &")
             ,@(map formal->variable out-formals)
             ,(sm:call (name (string-append "c.system." (code:event-name trigger)))
                       (arguments arguments))
+            ,(sm:call (name "c.perform"))
             ,(sm:call (name "c.match")
                       (arguments (list port-sm:return-string))))))))))
   (define (typed-provides-in->init trigger)
     (let* ((port (.port.name trigger))
            (event (.event.name trigger))
+           (formals (code:formal* trigger))
+           (formals (code:number-formals formals))
+           (arguments (map code:number-argument formals))
+           (out-formals (filter (negate ast:in?) formals))
            (port-event (simple-format #f "~a.~a" port event))
            (port-event-string (simple-format #f "~s" port-event))
            (port-return (simple-format #f "~a.return" port))
            (port-sm:return-string (simple-format #f "~s" port-return))
            (port-prefix (simple-format #f "~a." port))
            (event-slot (string-append "c.system." (code:event-name trigger)))
-           (formals (code:formal* trigger))
            (invoke (sm:code->string (sm:call (name event-slot)
-                                             (arguments (iota (length formals)))))))
+                                             (arguments arguments)))))
       (sm:generalized-initializer-list*
        port-event-string
        (sm:function
         (captures '("&"))
         (statement
          (sm:compound*
-          (sm:call (name "c.match")
-                   (arguments (list port-event-string)))
-          (sm:variable (type "std::string")
-                       (name "tmp")
-                       (expression
-                        (simple-format #f
-                                       "~s + dzn::to_string (~a)" port-prefix invoke)))
-          (sm:call (name "c.match")
-                   (arguments (list "tmp")))))))))
+          `(,(sm:call (name "c.match")
+                      (arguments (list port-event-string)))
+            ,@(map formal->variable out-formals)
+            ,(sm:variable (type "std::string")
+                          (name "tmp")
+                          (expression
+                           (simple-format #f
+                                          "~s + dzn::to_string (~a)" port-prefix invoke)))
+            ,(sm:call (name "c.perform"))
+            ,(sm:call (name "c.match")
+                      (arguments (list "tmp"))))))))))
   (define (void-requires-in->init trigger)
     (let* ((port (.port.name trigger))
            (port-return (simple-format #f "~a.return" port))
@@ -1550,7 +1580,7 @@ std::basic_ostream<Char, Traits> &")
           (sm:call (name "dzn::port_release")
                    (arguments
                     (list "c.dzn_locator"
-                          "&c.system"
+                          "nullptr"
                           (string-append "&" system-port))))))))))
   (define (typed-in->init port-pair) ;; FIXME: get rid of port-pair?
     (let* ((port (.port port-pair))
@@ -1565,8 +1595,8 @@ std::basic_ostream<Char, Traits> &")
           (sm:call (name "dzn::port_release")
                    (arguments
                     (list "c.dzn_locator"
-                          "&c.system"
-                          (simple-format #f "&c.system.~a" (.port port-pair)))))))))))
+                          "nullptr"
+                          (simple-format #f "&c.system.~a" port))))))))))
   (define (out->init trigger)
     (let* ((port (.port.name trigger))
            (event (.event.name trigger))
@@ -1597,19 +1627,30 @@ std::basic_ostream<Char, Traits> &")
                    (arguments (list flush-string)))
           (sm:statement*
            (simple-format #f "std::clog << ~s << std::endl" flush))
-          (sm:call (name "c.dzn_runtime.flush")
-                   (arguments (list "&c")))))))))
+          (sm:call
+           (name "c.dzn_runtime.flush")
+           (arguments
+            (list
+             (simple-format #f "c.system.~a.dzn_meta.require.component" port)
+             (sm:call (name "dzn::coroutine_id")
+                      (arguments (list "c.system.dzn_locator")))
+             "false")))))))))
   (sm:function
    (type "static std::map<std::string, std::function<void ()> >")
    (name "event_map")
    (formals (list (sm:formal (type (string-append container "&"))
-                             (name "c"))))
+                             (name "c"))
+                  (sm:formal (type "bool")
+                             (name "flush"))))
    (statement
     (sm:compound*
      (append
       (append-map provides->init (ast:provides-port* o))
       (append-map requires->init (ast:requires-port* o))
       (list
+       (sm:assign (variable (sm:call (name "c.dzn_runtime.performs_flush")
+                                     (arguments (list "nullptr"))))
+                  (expression "flush"))
        (sm:return*
         (sm:generalized-initializer-list
          (newline? #t)
@@ -1620,8 +1661,8 @@ std::basic_ostream<Char, Traits> &")
            (append
             (map void-provides-in->init (ast:provides-in-void-triggers o))
             (map typed-provides-in->init (ast:provides-in-typed-triggers o))
-            (map void-requires-in->init (code:requires-in-void-returns o))
-            (map typed-in->init (code:return-values o))
+            (map void-requires-in->init (code:blocking-requires-in-void-returns o))
+            (map typed-in->init (code:blocking-return-values o))
             (map out->init (ast:requires-out-triggers o))
             (map flush->init (ast:requires-port* o))
             (map flush->init (ast:provides-port* o)))))))))))))
@@ -1667,11 +1708,10 @@ std::basic_ostream<Char, Traits> &")
                         (arguments (list call-rdbuf))
                         ;; (arguments (list (sm:call (name "std::clog.rdbuf"))))
                         )))
-     (sm:call (name (simple-format #f "~a c" container))
-              (arguments '("flush")))
+     (sm:statement* (simple-format #f "~a c" container))
      (sm:call (name "connect_ports") (arguments '("c")))
      (sm:call (name "dzn::check_bindings") (arguments '("c.system")))
-     (sm:call (name "c") (arguments '("event_map (c)")))))))
+     (sm:call (name "c") (arguments '("event_map (c, flush)")))))))
 
 
 ;;;
