@@ -2,7 +2,7 @@
 ;;;
 ;;; Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2020, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
-;;; Copyright © 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2021, 2022, 2023, 2024 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -30,8 +30,8 @@
   #:use-module (dzn ast display)
   #:use-module (dzn ast goops)
   #:use-module (dzn ast)
-  #:use-module (dzn command-line)
   #:use-module (dzn config)
+  #:use-module (dzn command-line)
   #:use-module (dzn misc)
   #:use-module (dzn vm compliance)
   #:use-module (dzn vm goops)
@@ -55,8 +55,17 @@
 (define-method (process-external (pc <program-counter>))
   (let ((queues (.external-q pc)))
     (if (null? queues) '()
-        (let ((instances (map car queues)))
-          (append-map (cute run-external-q pc <>) instances)))))
+        (let* ((r:ports (map car queues))
+               (traces (append-map (cute run-external-q pc <>) r:ports)))
+          (define external-flush
+            (match-lambda
+              ((r:port trigger rest ...)
+               (let* ((r:component-port (runtime:other-port r:port))
+                      (instance (.container r:component-port))
+                      (trigger (trigger->component-trigger r:port trigger))
+                      (event (trigger->string trigger)))
+                 (append-map (cute run-requires-flush <> event) traces)))))
+          (append-map external-flush queues)))))
 
 (define (run-to-completion** pc event)
   (let* ((pc (clone pc #:instance #f #:trail '()))
@@ -64,7 +73,11 @@
     (cond
      ((eq? event 'defer)
       (if (null? (.defer pc)) '()
-          (run-defer-event pc event)))
+          (run-defer-event pc "<defer>")))
+     ((eq? event 'flush)
+      (if (or (pair? (.blocked pc)) (pair? (.defer pc))) '()
+          (let ((components (filter (is? <runtime:component>) (%instances))))
+            (append-map (cute run-flush pc <>) components))))
      ((eq? event 'external)
       (process-external pc))
      ((is-a? (%sut) <runtime:port>)
@@ -128,24 +141,65 @@ that PC has one more collaterally blocked coroutine on the same port."
          (state-number-table (make-hash-table))
          (state-number-count (list 1))
          (pc->state-number (pc->state-number state-number-table
-                                             state-number-count)))
+                                             state-number-count))
+         (model ((compose .type .ast %sut)))
+         (defers (append-map
+                  (cute tree-collect-filter
+                        (negate (disjoin (is? <expression>)
+                                         (is? <location>)))
+                        (is? <defer>)
+                        <>)
+                  (ast:model** model)))
+         (defer? (pair? defers))
+         (external? (and (not (is-a? (%sut) <runtime:port>))
+                         (pair? (ast:external-port* model))))
+         (synthetic-labels `(,@(if defer? '(defer) '())
+                             flush
+                             ,@(if external? '(external) '()))))
+
+    (define* (follow-blocked-unmemoized alist pc #:key (seen '()))
+      (let* ((to (pc->state-number pc))
+             (trace (any
+                     (match-lambda
+                       ((from pc traces ...)
+                        (find
+                         (conjoin
+                          pair?
+                          (compose pair? .blocked car)
+                          (compose (cute eq? <> to)
+                                   pc->state-number car)
+                          (compose not (cute memq <> seen)
+                                   pc->state-number last))
+                         traces)))
+                     alist))
+             (pc' (and=> trace last))
+             (from (and=> pc' pc->state-number))
+             (trace (or trace '())))
+        (if (or (not from) (eq? from 1)) trace
+            (let ((seen (cons from seen))
+                  (tail (follow-blocked alist pc' #:seen seen)))
+              (append trace tail)))))
+
+    (define follow-blocked
+      (let ((cache (make-hash-table)))
+        (lambda* (alist pc #:key (seen '()))
+          (let ((to (pc->state-number pc)))
+            (or (hashq-ref cache to)
+                (let ((trace (follow-blocked-unmemoized alist pc #:seen seen)))
+                  (hashq-set! cache to trace)
+                  trace))))))
 
     (define (blocked-prefix pc)
-      (let* ((blocking-pc (rtc-block-pc (cdar (.blocked pc))))
-             (blocking-pc (.previous blocking-pc))
-             (blocked-from (and=> blocking-pc pc->state-number))
-             (pc+traces (hash-ref lts blocked-from)))
-        (match pc+traces
-          ((pc . traces)
-           (or (find (compose pair? .blocked car) traces)
-               '()))
-          (_
-           '()))))
+      (let ((to (pc->state-number pc)))
+        (if (= to 1) '()
+            (let ((alist (hash-table->alist lts)))
+              (follow-blocked alist pc #:seen (list to))))))
 
     (define (check-compliance-blocked trace)
       (let ((orig-pc (last trace)))
         (let* ((blocked-prefix (blocked-prefix orig-pc))
                (q-trigger (and (null? blocked-prefix)
+                               (not (is-a? (%sut) <runtime:system>))
                                (and=>
                                 (as (.q (get-state orig-pc (%sut))) <pair>)
                                 car)))
@@ -164,18 +218,21 @@ that PC has one more collaterally blocked coroutine on the same port."
                                                        full-trace)))
                (illegal? (is-status? <implicit-illegal-error>))
                (checked (filter (compose not illegal? car) checked))
-               (pcs (map car checked))
-               (corrected (if (not (= 1 (length pcs))) trace
-                              (cons (car pcs) trace))))
-          corrected)))
+               (prefix-length (length blocked-prefix))
+               (drop-prefix? (and (pair? checked)
+                                  (> (length (car checked)) prefix-length))))
+          (if (not drop-prefix?) trace
+              (drop-right (car checked) prefix-length)))))
 
     (define (run-label orig-pc label)
-      (%debug "run-label ~a\n" label)
+      (%debug (current-source-location) "run-label ~a" label)
       (let* ((orig-pc (clone orig-pc #:collateral-instance #f))
              (defer? (eq? label 'defer))
-             (sw-pc (if defer? orig-pc
+             (port (and (string? label) (.port (string->trigger label))))
+             (switch? (not defer?))
+             (sw-pc (if (not switch?) orig-pc
                         (switch-context orig-pc)))
-             (bob-pc (if defer? orig-pc
+             (bob-pc (if (not switch?) orig-pc
                          (blocked-on-boundary-switch-context orig-pc)))
              (traces (cond
                       ((and (eq? orig-pc sw-pc)
@@ -193,23 +250,28 @@ that PC has one more collaterally blocked coroutine on the same port."
                                               (append-map run-flush flush))))
                          (append traces
                                  (run-to-completion** orig-pc label))))))
-             (blocked? (not (eq? bob-pc orig-pc)))
-             (skip-compliance? (or (.skip-compliance? orig-pc)
-                                   (is-a? (%sut) <runtime:port>)
-                                   (null? traces)))
+             (blocked (.blocked orig-pc))
+             (blocked? (pair? blocked))
+             (blocked? (or blocked?
+                           (not (eq? bob-pc orig-pc))))
+             (double-blocked? (find (compose pair? .blocked)
+                                    (map cdr blocked)))
+             (skip-compliance? (or (is-a? (%sut) <runtime:port>)
+                                   (null? traces)
+                                   double-blocked?))
              (traces (map (cute append <> (list orig-pc)) traces))
              (traces (cond
-                      ((and (not skip-compliance?) (not blocked?))
+                      ((and (not skip-compliance?)
+                            (not blocked?))
                        (let* ((cpc orig-pc)
                               (cpc (reset-replies cpc))
                               (cpc (clone cpc #:instance #f)))
                          (check-provides-compliance* cpc label traces)))
                       ((and blocked?
-                            (not (.skip-compliance? orig-pc))
+                            (not skip-compliance?)
                             (not (eq? label 'defer))
                             (not (or (.status pc)
-                                     (is-a? (%sut) <runtime:port>)
-                                     (is-a? (%sut) <runtime:system>))))
+                                     (is-a? (%sut) <runtime:port>))))
                        (map check-compliance-blocked traces))
                       (else
                        traces)))
@@ -217,15 +279,20 @@ that PC has one more collaterally blocked coroutine on the same port."
                          (filter-implicit-illegal-only traces)))
              (traces (filter (compose not (is-status? <blocked-error>) car)
                              traces))
-             (traces (filter (compose not (cute collateral? <> pc) car) traces)))
-        (map (cute rewrite-trace-head prune-defer <>) traces)))
+             (traces (filter (compose not (cute collateral? <> pc) car) traces))
+             (traces (map (cute rewrite-trace-head prune-defer <>) traces)))
+        traces))
 
     (define (run-labels pc labels)
-      (%debug "run-labels ~a\n" labels)
+      (%debug (current-source-location) "run-labels ~a" labels)
       (let loop ((labels labels) (traces '()))
         (if (null? labels) traces
             (let* ((label (car labels))
-                   (new (run-label pc label)))
+                   (cr-pc (blocked-on-boundary-collateral-release pc))
+                   (cr? (not (eq? cr-pc pc)))
+                   (run-rtc? (and cr? (not (return-trigger? label))))
+                   (new (if run-rtc? (run-to-completion cr-pc 'rtc)
+                            (run-label pc label))))
               (cond
                ((find (compose (is-status? <livelock-error>) car) new)
                 (format (current-error-port) "warning: livelock, bailing out\n")
@@ -236,11 +303,11 @@ that PC has one more collaterally blocked coroutine on the same port."
     (hash-set! state-number-table "<illegal>" 0)
     (let loop ((pc pc))
       (let ((from (pc->state-number pc)))
-        (%debug "loop ~a~a\n" from (if (hash-ref lts from) ": seen!" ""))
+        (%debug (current-source-location) "loop ~a~a" from (if (hash-ref lts from) ": seen!" ""))
         (unless (or (.status pc) (hash-ref lts from))
           (let* ((labels (labels pc))
                  (labels (if (is-a? (%sut) <runtime:port>) labels
-                             (cons* 'defer 'external labels)))
+                             (append synthetic-labels labels)))
                  (traces (run-labels pc labels)))
             (when (or (is-a? (%sut) <runtime:port>)
                       (any pair? traces))
@@ -630,7 +697,8 @@ RTC-LTS->LTS."
     (when (> (dzn:debugity) 1)
       (ast:pretty-print root (current-error-port)))
     (parameterize ((%compliance-check? #f)
-                   (%debug? (> (dzn:debugity) 0))
+                   (%debug? (and (not (zero? (dzn:debugity)))
+                                 (dzn:debugity)))
                    (%exploring? #t)
                    (%queue-size queue-size)
                    (%queue-size-defer queue-size-defer)
@@ -660,8 +728,8 @@ RTC-LTS->LTS."
          (model (ast:get-model root (ast:dotted-name model))))
     (when (> (dzn:debugity) 1)
       (ast:pretty-print root (current-error-port)))
-    (parameterize ((%compliance-check? #f)
-                   (%debug? (> (dzn:debugity) 0))
+    (parameterize ((%debug? (and (not (zero? (dzn:debugity)))
+                                 (dzn:debugity)))
                    (%exploring? #t)
                    (%queue-size queue-size)
                    (%queue-size-defer queue-size-defer)
