@@ -3,7 +3,7 @@
 ;;; Copyright © 2018, 2019 Henk Katerberg <hank@mudball.nl>
 ;;; Copyright © 2018, 2019, 2020, 2021, 2022, 2023 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2021, 2022, 2023, 2024 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -44,13 +44,15 @@
   #:use-module (srfi srfi-26)
   #:use-module (srfi srfi-43)
 
+  #:use-module (ice-9 curried-definitions)
   #:use-module (ice-9 match)
+  #:use-module (ice-9 peg)
   #:use-module (ice-9 poe)
   #:use-module (ice-9 rdelim)
   #:use-module (ice-9 regex)
 
+  #:use-module (dzn command-line)
   #:use-module (dzn misc)
-  #:use-module (dzn peg)
   #:export (%<declarative-illegal>
             %<illegal>
             add-failures
@@ -62,6 +64,7 @@
             cleanup-aut
             cleanup-error
             display-lts
+            edge-from
             edge-label
             edge-to
             edge?
@@ -71,12 +74,14 @@
             lts->rtc-lts
             lts->traces
             lts-hide
+            lts-hide-state
             lts->tags
             make-shared-string
             node?
             node-edges
             node-state
-            remove-illegal))
+            remove-illegal
+            remove-state-loops))
 
 ;;; TODO:
 ;;; * functional style
@@ -158,7 +163,7 @@
 
 (define (edge-canonical-label edge)
   (let ((label (edge-label edge)))
-    (if (string-prefix? "<state>" label) %<state> label)))
+    (if (string-contains label %<state>) %<state> label)))
 
 (define-immutable-record-type <node>
   (make-node state edges pred initial? color parent distance cycle)
@@ -236,11 +241,11 @@
   (let ((tau (map make-shared-string tau))
         (exclude-tau (map make-shared-string exclude-tau)))
     (define (set-edge-tau edge)
-      (let ((tau? (and (or (memq (edge-label edge) tau)
+      (let ((tau? (and (not (memq (edge-label edge) exclude-tau))
+                       (or (memq (edge-label edge) tau)
                            (find (cut string-prefix? <> (edge-label edge))
                                  (append (map (cute string-append <> ".") tau)
                                          (map (cute string-append <> "(") tau))))
-                       (not (memq (edge-label edge) exclude-tau))
                        #t)))
         (set-field edge (edge-tau?) tau?)))
     (define (set-edge-taus node)
@@ -356,10 +361,33 @@
                                 (trace lts (car livelock-nodes))))
          (loop-entry-node (and loop-entry-trace (car livelock-nodes)))
          (loop-trace (and loop-entry-node (tau-loop loop-entry-node lts))))
-    (if (null? livelock-nodes) #f
-        (append (or loop-entry-trace '())
-                (list (make-edge-loop))
-                (or loop-trace '())))))
+    (and (pair? livelock-nodes)
+         (append (or loop-entry-trace '())
+                 (list (make-edge-loop))
+                 (or loop-trace '())))))
+
+
+;;;
+;;; State
+;;;
+
+(define (lts-hide-state lts)
+  (define (set-state-edge-tau edge)
+    (if (not (string-contains (edge-label edge) %<state>)) edge
+        (set-field edge (edge-label) %tau)))
+  (define (set-state-edge-taus node)
+    (let ((edges (map set-state-edge-tau (node-edges node))))
+      (set-field node (node-edges) edges)))
+  (vector-map-one set-state-edge-taus lts))
+
+(define (remove-state-loops lts)
+  (define (state-edge? edge)
+    (and (string-contains (edge-label edge) %<state>)
+         (= (edge-from edge) (edge-to edge))))
+  (define (remove-state node)
+    (let ((edges (filter (negate state-edge?) (node-edges node))))
+      (set-field node (node-edges) edges)))
+  (vector-map-one remove-state lts))
 
 
 ;;;
@@ -388,17 +416,10 @@
 ;;;
 ;;; Deadlock.
 ;;;
-(define (remove-state-edges lts)
-  (define (state-edge? edge)
-    (string-prefix? "<state>" (edge-label edge)))
-  (define (remove-state node)
-    (let ((edges (filter (negate state-edge?) (node-edges node))))
-      (set-field node (node-edges) edges)))
-  (vector-map-one remove-state lts))
 
 (define (deadlock-nodes lts)
   "States without outgoing edges"
-  (let* ((lts (remove-state-edges lts))
+  (let* ((lts (remove-state-loops lts))
          (lts (annotate-exclude lts (list %<declarative-illegal>))))
     (define (edges? state)
       (let ((node (vector-ref lts state)))
@@ -552,8 +573,7 @@ from LABELS."
         (= (length ports)
            (length provides-ports))))
     (and (idle-node? node)
-         (not (find (compose (disjoin (cute eq? %<ack> <> )
-                                      (cute eq? %<defer> <>))
+         (not (find (compose (cute memq <> (list %<ack> %<defer>))
                              edge-label)
                     (node-edges node)))))
 
@@ -590,19 +610,26 @@ from LABELS."
       (if (memq label (list %tau %<ack>)) trace
           (append trace (list label))))
 
+    (define (illegal-node? node)
+      (let ((edges (node-edges node)))
+        (find (compose (cute eq? <> %<illegal>) edge-label) edges)))
+
     (define (trace-close trace index)
-      (let* ((node (vector-ref lts index))
-             (close-edge (node-close node)))
-        (if (or (node-allowed-end? node)
-                (not close-edge))
-            trace
-            (let ((ext-trace (trace-extend trace (edge-label close-edge))))
-              (trace-close ext-trace (edge-to close-edge))))))
+      (if (= index (vector-length lts)) trace
+          (let* ((node (vector-ref lts index))
+                 (close-edge (or (node-close node)
+                                 (illegal-node? node)))
+                 (closed? (and (or (node-allowed-end? node)
+                                   (not close-edge))
+                               (not (illegal-node? node)))))
+            (if closed? trace
+                (let ((ext-trace (trace-extend trace (edge-label close-edge))))
+                  (trace-close ext-trace (edge-to close-edge)))))))
 
     (define (trace-log trace count)
-      (let ((file-name (format #f "~a~a.~a" (or dir "") base count)))
+      (let ((file-name (simple-format #f "~a~a.~a" (or dir "") base count)))
         (when verbose?
-          (format (current-error-port) "~a\n" file-name))
+          (simple-format (current-error-port) "~a\n" file-name))
         (let ((port (if dir (open-output-file file-name)
                         (current-output-port))))
           (display (string-join trace "\n" 'suffix) port))))
@@ -611,6 +638,7 @@ from LABELS."
       (let ((generated-trace? #f))
         (and
          (not (hashq-ref done index #f))
+         (< index (vector-length lts))
          (let ((node (vector-ref lts index)))
            (hashq-set! done index #t)
            (let loop ((edges (node-edges node)) (generated-trace? #f))
@@ -621,11 +649,14 @@ from LABELS."
                     ((= edge-index index)
                      (loop (cdr edges) generated-trace?))
                     (else
-                     (let* ((ext-trace (trace-extend trace (edge-label edge)))
-                            (trace? (not (step edge-index ext-trace))))
-                       (when trace?
-                         (trace-log (trace-close ext-trace edge-index) count)
-                         (set! count (1+ count)))
+                     (let* ((trace (trace-extend trace (edge-label edge)))
+                            (trace? (not (step edge-index trace)))
+                            (closed? (= edge-index (vector-length lts))))
+                       (when (not (step edge-index trace))
+                         (let ((trace (if closed? trace
+                                          (trace-close trace edge-index))))
+                           (trace-log trace count)
+                           (set! count (1+ count))))
                        (loop (cdr edges) #t)))))))))))
 
     (annotate)
@@ -659,16 +690,32 @@ from LABELS."
             (edges (map convert-edge (node-edges node))))
         (set-fields node ((node-pred) pred) ((node-edges) edges))))
 
+    (define (illegal-node? node)
+      (let* ((edges (node-edges node))
+             (labels (map edge-label edges)))
+        (memq %<illegal> labels)))
+
+    (define ((illegal-node->deadlock deadlock) i node)
+      (if (not (illegal-node? node)) node
+          (let* ((edges (node-edges node))
+                 (edge (car edges))
+                 (edge (set-field edge (edge-to) deadlock)))
+            (set-field node (node-edges) (list edge)))))
+
+    (define (illegal-livelock->deadlock lts)
+      (let ((deadlock (vector-length lts)))
+        (vector-map (illegal-node->deadlock deadlock) lts)))
+
     (let* ((text (string-join data "\n"))
            (lts (aut-text->lts text #:pred? #t))
            (lts (vector-map-one convert-node lts))
-           (lts (if illegal? lts (remove-illegal lts)))
-           (lts (remove-state-edges lts))
+           (lts (if illegal? (illegal-livelock->deadlock lts)
+                    (remove-illegal lts)))
            (initial (initial lts))
            (base (string-append model ".trace"))
            (dir (cond ((equal? out "-") #f)
                       ((equal? out "") "./")
-                      (else (format #f "~a/" out)))))
+                      (else (simple-format #f "~a/" out)))))
       (generate-traces initial lts provides-ports provides-in dir base
                        #:verbose? verbose?))))
 
@@ -685,16 +732,12 @@ from LABELS."
         ("declarative_illegal" #t)
         (_ #f)))
 
-    (define (modeling? o)
+    (define (state? o)
       (and (string? o)
-           (string-contains o "'internal(")))
-
-    (define (trigger? o)
-      (and (string? o)
-           (string-contains o "'in(")))
+           (string-contains o "'state(")))
 
     (define (rtc? node)
-      (find (disjoin modeling? trigger?) (map edge-label (node-edges node))))
+      (find state? (map edge-label (node-edges node))))
 
     (define (extend trail label)
       (match label
@@ -733,27 +776,30 @@ from LABELS."
 ;;;
 (define (parse-label label)
   (define-peg-string-patterns
-    "tree               <-- event / modeling / defer-qout / tag / reply / state / return / queue / tau-literal / illegal / error / end / flush / blocking / parse-error
+    "tree               <-- event / modeling / defer-qout / tag / reply / state / tau-void / tau-reply / return / queue / tau-literal / illegal / error / end / flush / blocking / parse-error
      parse-error        <-- [a-zA-Z_0-9'()]*
      event              <-- port-name tick direction lpar scope* action-literal lpar scope* direction tick event-name rpar rpar
      modeling           <-- port-name tick internal-literal lpar scope* ('inevitable' / 'optional') rpar
      queue              <-- port-name tick queue-direction lpar scope* action-literal lpar scope* direction tick event-name rpar rpar
      end                <   scope* ('end' / 'silent_end')
      return             <-- 'return'
-     flush              <-- port-scope* identifier tick 'flush'
-     blocking           <-- port-scope* identifier tick 'blocking'
+     flush              <-- port-scope* identifier tick flush-literal
+     blocking           <-- port-scope* identifier tick port- 'blocking'
      reply              <-- port-name tick reply-literal lpar scope* reply-value rpar
-     state              <-- port-name tick state-literal state-arguments
-     state-arguments    <-  (lpar state-argument (comma state-argument)* rpar)?
+     tau-reply          <-- port-name tick tau-reply-literal lpar scope* reply-value rpar
+     state              <-- port-name tick state-literal lpar port-name tick state-arguments rpar
+     state-arguments    <-- void-literal / variables-literal (lpar state-argument (comma state-argument)* rpar)
      state-argument     <-- bool / int / enum-literal
      scope              <   identifier tick
      tag                <   tag-literal lpar int comma int rpar
+     port-              <   'port_'
      port-name          <-  port-scope* identifier
-     port-scope         <   scope !(internal-literal / queue-direction / direction / reply-literal / state-literal / 'queue_full' / 'flush' / 'blocking')
+     port-scope         <   scope !(void / void-literal / internal-literal / queue-direction / direction / reply-literal / tau-reply-literal / state-literal / variables-literal / port-? 'queue_full' / port-? 'flush' / port- 'blocking')
      event-name         <-  identifier
      reply-value        <-  bool-literal lpar bool rpar / lpar enum-literal rpar / int-literal lpar int rpar / void-literal lpar void rpar
      bool-literal       <   'Bool'
      bool               <-- ('true' / 'false' )
+     flush-literal      <   port-? 'flush'
      int-literal        <   'Int'
      int                <-- '-'?[0-9]+
      void-literal       <   'Void'
@@ -762,19 +808,22 @@ from LABELS."
      enum-literal       <-- (enum tick)* enum-field
      enum               <-  identifier
      enum-field         <-  identifier
-     direction          <   ('qin' / 'in' / 'out') !identifier
+     direction          <   (port-? 'in' / port-? 'out' / port- 'qin') !identifier
      defer-qout         <-- 'defer_qout' paren-arguments
      paren-arguments    <   lpar ((!(lpar / rpar) .)* paren-arguments*)* rpar
-     queue-direction    <-- 'qout'
+     queue-direction    <-- port-? 'qout'
      action-literal     <   'action'
-     internal-literal   <   'internal' / 'silent'
-     reply-literal      <   'reply' (tick 'reordered')?
-     state-literal      <   'state'
+     internal-literal   <   port-? 'internal'
+     reply-literal      <   port-? 'reply' (tick 'reordered')?
+     tau-reply-literal  <   'tau_reply'
+     state-literal      <   port-? 'state'
+     variables-literal  <   'variables'
      tag-literal        <   'tag'
      tau-literal        <   'tau'
-     illegal            <-- 'illegal' / 'declarative_illegal' / 'constrained_illegal'
+     tau-void           <   'tau_void'
+     illegal            <-- 'illegal' / 'declarative_illegal' / 'constrained_illegal' / 'non_compliance'
      error              <--  queue-full / range-error / reply-error / missing-reply / second-reply
-     queue-full         <-  'queue_full' / port-name tick 'queue_full'
+     queue-full         <-  'queue_full' / port-name tick port- 'queue_full'
      range-error        <-  'range_error'
      reply-error        <-  'double_reply_error' / 'no_reply_error'
      missing-reply      <-  'missing_reply'
@@ -784,19 +833,24 @@ from LABELS."
      rpar               <   [)]
      comma              <   [,][ ]*
      identifier         <-- &(direction [a-zA-Z0-9_]+) [a-zA-Z0-9_]+ / !direction [a-zA-Z_][a-zA-Z0-9_]*")
-  (let* ((match (match-pattern tree label))
-         (end (peg:end match))
-         (tree (peg:tree match)))
-    (if (eq? (string-length label) end)
-        (if (symbol? tree) '()
-            (cdr tree))
-        (if match
-            (begin
-              (format (current-error-port) "input: ~a\nparse error: at offset: ~a\n~s\n" label end tree)
-              #f)
-            (begin
-              (format (current-error-port) "parse error: no match\n")
-              #f)))))
+  (let* ((match? (match-pattern tree label))
+         (end (peg:end match?))
+         (tree (peg:tree match?))
+         (equal-length? (eq? (string-length label) end)))
+    (cond
+     ((and equal-length?
+           (symbol? tree))
+      '())
+     (equal-length?
+      (cdr tree))
+     (match?
+      (simple-format (current-error-port)
+                     "input: ~a\nparse error: at offset: ~a\n~s\n"
+                     label end tree)
+      #f)
+     (else
+      (simple-format (current-error-port) "parse error: no match\n")
+      #f))))
 
 (define (cleanup-error e)
   (string-append "<" (string-map (lambda (c) (if (eq? c #\_) #\- c)) e) ">"))
@@ -807,11 +861,12 @@ from LABELS."
       (string-join (map helper parameters) ",")))
   (define (helper tree)
     (match tree
-      (('parse-error parse-error) (format (current-error-port) "parse error:~s\n" tree) parse-error)
+      (('parse-error parse-error) (simple-format (current-error-port) "parse error:~s\n" tree) parse-error)
       (('defer-qout label) "<defer>")
       (('error error) (cleanup-error error))
       (('error ('identifier port) error) (cleanup-error error))
       (('event ('identifier port) ('identifier event)) (string-append port "." event))
+      (('flush ('identifier port)) (string-append port ".<flush>"))
       (('flush ('identifier port) "flush") (string-append port ".<flush>"))
       (('blocking ('identifier port) "blocking") (string-append port ".<blocking>"))
       (('illegal illegal) (and illegal? (cleanup-error illegal)))
@@ -823,8 +878,12 @@ from LABELS."
       (('reply ('identifier port) ('enum-literal scope ... ('identifier name) ('identifier field))) (string-append port "." name ":" field))
       (('reply ('identifier port) ('enum-literal (scope ... ('identifier name)) ('identifier field))) (string-append port "." name ":" field))
       (('return return) (and internal? "return"))
-      (('state ('identifier port) parameters) (string-append port ".<state>(" (helper-params parameters) ")"))
-      (('state ('identifier port)) (string-append port ".<state>"))
+      (('state ('identifier port) parameters) (string-append port "." %<state> (helper-params parameters) ")"))
+      (('state ('identifier port)) (string-append port "." %<state>))
+      (('state ('identifier port) ... 'state-arguments) (string-append (car port) "." %<state>))
+      (('state ('identifier port) ('identifier interface) ('state-arguments arguments ..1)) (string-append port "." %<state> "(" (helper arguments) ")"))
+      (('state-arguments ('identifier interface) arguments) (helper arguments))
+      ((('state-argument value) ..1) (string-join (map helper value) ","))
       (('state-argument value) (helper value))
       (('bool value) value)
       (('int value) value)
@@ -841,10 +900,11 @@ from LABELS."
        (define (drop-prefix o)
          (if (and prefix (string-prefix? prefix o)) (substring o prefix-length)
              o))
-       (format #f "\"~a\""
-               (drop-prefix (cleanup-label (symbol->string label)
-                                           #:illegal? illegal?
-                                           #:internal? internal?)))))))
+       (simple-format
+        #f "\"~a\""
+        (drop-prefix (cleanup-label (symbol->string label)
+                                    #:illegal? illegal?
+                                    #:internal? internal?)))))))
 
 
 ;;;
@@ -869,22 +929,24 @@ from LABELS."
         (loop (read-line input-port 'concat))))))
 
 (define* (display-lts lts #:key (separator "\n") (port (current-output-port)))
-  (let* ((edges (append-map node-edges (vector->list lts)))
-         (header (format #f "des (~a,~a,~a)"
-                         (initial lts)
-                         (length edges)
-                         (vector-length lts)))
-         (lines (map
-                 (lambda (e)
-                   ;; format has a significant performance impact on
-                   ;; large LTSs.
-                   (string-append
-                    "("
-                    (number->string (edge-from e)) ","
-                    "\"" (edge-label e) "\","
-                    (number->string (edge-to e))
-                    ")"))
-                 edges))
-         (lines (cons header lines))
-         (text (string-join lines separator)))
-    (display text port)))
+  (define (display-edge edge)
+    (display
+     (string-append
+      "("
+      (number->string (edge-from edge))
+      ",\""
+      (let ((label (edge-label edge)))
+        (if (pair? label) (string-join label)
+            label))
+      "\","
+      (number->string (edge-to edge))
+      ")"
+      separator)
+     port))
+  (let ((edges (append-map node-edges (vector->list lts))))
+    (simple-format port "des (~a,~a,~a)~a"
+                   (initial lts)
+                   (length edges)
+                   (vector-length lts)
+                   separator)
+    (for-each display-edge edges)))
