@@ -188,8 +188,8 @@
                         o 'trigger #:event-predicate event-predicate))))
     (sort (append (if (and (null? statements) trigger-complete?) '("illegal")
                       '())
-           triggers)
-     string<)))
+                  triggers)
+          string<)))
 
 (define* (complete:action-names o #:key (event-predicate identity))
   (cond ((slot o 'component) (complete:port-event-names o 'action #:event-predicate event-predicate))
@@ -431,7 +431,7 @@
                       (else
                        '())))
          (statements (if illegal? (cons "illegal" statements) statements))
-         (statements (cons "if" statements))
+         (statements (cons* "defer" "if" statements))
          (statements (if else? (cons "else" statements) statements))
          (actions    (complete:action-names context))
          (types      (complete:type-names context))
@@ -476,9 +476,9 @@
                                     (cute string-append dir "/" <>))))))))
 
 (define (strip-prefix prefix string)
-           (if (and (string? string) (string-prefix? prefix string))
-               (substring string (string-length prefix))
-               string))
+  (if (and (string? string) (string-prefix? prefix string))
+      (substring string (string-length prefix))
+      string))
 
 (define* (complete:imports o context #:key (imports '()))
   (let* ((file-name (.file-name o))
@@ -834,11 +834,22 @@
     ((? (is? 'unknown-identifier))
      (complete:root (parent context tree?)
                     (context:parent context tree?) offset #:debug? debug?))
-    ((and (? complete?) (? (negate (is? 'comment))))
-     '())
-    ((and (? (is? 'or)) (? incomplete?))
-     (cond ((context:parent context 'on)
-            => complete:trigger-names)))
+    ((and (? (is? 'or)) (? incomplete?)) (=> failure)
+     (let* ((context (or (context:parent context 'compound)
+                         (context:parent context 'on)))
+            (triggers (and=> (is-a? (.tree context) 'on)
+                             tree:trigger*)))
+       (cond
+        ((is-a? context 'compound)
+         => (cute complete:statements <> offset))
+        ((and (pair? triggers)
+              (every complete? triggers)
+              context)
+         => (cute complete:statements <> offset))
+        ((is-a? (.tree context) 'on)
+         (complete:trigger-names context))
+        (else
+         (failure)))))
     (_
      (or (and (pair? o)
               (and=> (find incomplete? (cdr o))
