@@ -177,10 +177,4 @@
 (define-templates main-out-arg code:main-out-arg main-out-arg-grammar)
 (define-templates main-event-map-void ast:void-in-triggers event-map-prefix)
 (define-templates main-event-map-typed ast:typed-in-triggers event-map-prefix)
-(define-templates main-event-map-out ast:requires-out-triggers event-map-prefix)
-(define-templates main-event-map-void-provides-in ast:provides-in-void-triggers event-map-prefix)
-(define-templates main-event-map-typed-provides-in ast:provides-in-typed-triggers event-map-prefix)
-(define-templates main-event-map-void-requires-in code:requires-in-void-returns event-map-prefix)
-(define-templates main-event-map-typed-in code:return-values event-map-prefix)
 (define-templates main-event-map-flush ast:requires-port* event-map-prefix)
-(define-templates main-event-map-match-return code:main-event-map-match-return)
