@@ -1,9 +1,9 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2016, 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2016, 2018, 2019, 2020, 2021, 2022, 2024 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018, 2019 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2018 Henk Katerberg <hank@mudball.nl>
-;;; Copyright © 2018, 2021, 2022, 2023 Rutger (regtur) van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2018, 2021, 2022, 2023, 2024, 2025 Rutger (regtur) van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2018, 2020, 2021, 2022, 2023 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2017, 2018 Johri van Eerd <vaneerd.johri@gmail.com>
 ;;;
@@ -129,10 +129,13 @@ actions."
          (taus (delete-duplicates
                 (append-map events-trigger/action
                             (append out-triggers in-actions))))
+         (flush-taus (map (compose (cute string-append <> ".<flush>")
+                                   makreel:.name)
+                          (ast:port* model)))
          (state-taus (map (compose (cute string-append <> ".<state>")
                                    makreel:.name)
                           (ast:port* model)))
-         (taus `("tag" "<defer>" ,@state-taus ,@taus)))
+         (taus `("tag" "<defer>" ,@flush-taus ,@state-taus ,@taus)))
     (string-join taus ",")))
 
 (define (deterministic-labels component)
@@ -210,7 +213,8 @@ actions."
                  (loop (get-input in-out.pipeline format))))))))
 
 (define root+model->makreel
-  (perfect-funcq 1024
+  (perfect-funcq
+   1024
    (lambda (root model)
      (with-output-to-string (cute makreel:model->makreel root model)))))
 
@@ -273,7 +277,7 @@ actions."
 (define (in-out:aut->aut-weak-trace options)
   (let* ((model (options-model options))
          (taus (if (not (is-a? model <interface>)) '()
-                   '("--tau=inevitable,optional,tag,<flush>")))
+                   '("--tau=inevitable,optional,tag")))
          (timings? (dzn:command-line:get 'timings)))
     `("ltsconvert" ,(if timings? "--timings" "--quiet")
       "-eweak-trace" ,@taus "--in=aut" "--out=aut")))
@@ -282,7 +286,7 @@ actions."
   (let* ((model (options-model options))
          (name (makreel:full-name model))
          (taus (if (not (is-a? model <interface>)) '()
-                   `(,(format #f "--tau=~aflush,~ainternal,tag" name name))))
+                   `(,(format #f "--tau=~ainternal,tag" name))))
          (timings? (dzn:command-line:get 'timings)))
     `("ltsconvert" ,(if timings? "--timings" "--quiet")
       "-eweak-trace" ,@taus "--in=aut" "--out=aut")))
@@ -413,9 +417,14 @@ for MODEL, using ROOT."
     (define (flags->string)
       (string-append
        (imports->string)
+       (if (eq? (%queue-size) %default-queue-size) ""
+           (format #f " --queue-size=~a" (%queue-size)))
+       (if (eq? (%queue-size-defer) %default-queue-size-defer) ""
+           (format #f " --queue-size-defer=~a" (%queue-size-defer)))
+       (if (eq? (%queue-size-external) %default-queue-size-external) ""
+           (format #f " --queue-size-external=~a" (%queue-size-external)))
        (if (%no-unreachable?) " --no-unreachable" "")
-       (flag->string 'no-constraint)
-       (flag->string 'no-non-compliance)))
+       (flag->string 'no-constraint)))
     (let ((file-name (ast:source-file root))
           (model-name (makreel:unticked-dotted-name model)))
       (match command
@@ -552,9 +561,10 @@ init for MODEL unless INIT."
 
 (define (report-fail model assert trace)
   (define (remove-flushes trace)
-    (filter (negate (cut string-contains <> "<flush>")) trace))
+    (filter (negate (cute string-contains <> "<flush>")) trace))
   (define (drop-queue-full-tail trace)
-    (append (take-while (negate (cut equal? "<queue-full>" <>)) trace) (list "<queue-full>")))
+    (append (take-while (negate (cute equal? "<queue-full>" <>)) trace)
+            (list "<queue-full>")))
   (define (tag->message tag)
     (apply format #f "~a:~a:~a: error: code will never be executed"
            (.file-name (.location (.behavior model)))
@@ -638,42 +648,21 @@ init for MODEL unless INIT."
 ;;;
 
 (define (reduce-or all? l)
-  (if all? (fold (cut or <> <>) #f (map (cut <>) l))
+  (if all? (fold (cut or <> <>) #f (map (cute <>) l))
       (fold (lambda (e res) (or res (e))) #f l)))
 
-(define (mcrl2:verify-interface-asserts model root)
-  (and
-   (not (command-line:get 'no-interfaces))
-   (let* ((model-name (makreel:unticked-dotted-name model))
-          (asserts (verify-pipeline "verify-interface" root model))
-          (asserts (result-split asserts))
-          (nondets (verify-pipeline "verify-interface-nondet" root model))
-          (nondets (result-split nondets))
-          (lts-tags (get-tags asserts))
-          (unreachable (assert-unreachable lts-tags (model-tags model)))
-          (result `(,unreachable
-                    ,@asserts
-                    ,@nondets))
-          (deadlock? (get-trace 'deadlock result)))
-     (define* (report-assert assert #:key skip?)
-       (report assert skip? (get-trace assert result) model))
-     (reduce-or (command-line:get 'all)
-                `(,(cute report-assert 'deadlock)
-                  ,@(if (%no-unreachable?) '()
-                        `(,(cut report-assert 'unreachable #:skip? deadlock?)))
-                  ,(cute report-assert 'livelock)
-                  ,(cute report-assert 'deterministic))))))
-
 (define (mcrl2:verify-compliance root model)
   (let* ((output status (verify-pipeline "verify-compliance" root model))
          (lines (and output (string-split output #\newline)))
-         (stdout-status (and lines (filter (cut string-prefix? "result: " <>) lines)))
+         (stdout-status (and lines
+                             (filter (cute string-prefix? "result: " <>) lines)))
          (stdout-status (and (pair? stdout-status) (car stdout-status)))
          (status (if (and (zero? status)
                           stdout-status
                           (string=? stdout-status "result: true"))
                      0 1))
-         (trace (and lines (find (cut string-prefix? "counter_example_weak_failures_refinement: " <>) lines)))
+         (trace (and lines
+                     (find (cute string-prefix? "counter_example_weak_failures_refinement: " <>) lines)))
          (trace (and trace (substring trace (1+ (string-index trace #\:)))))
          (trace (and trace (string-trim-both trace)))
 
@@ -681,7 +670,8 @@ init for MODEL unless INIT."
          (trace (and trace (hide-illegal-labels trace)))
 
          (trace (and trace (if (string-null? trace) trace (string-append trace "\n"))))
-         (component-accepts (and lines (find (cut string-prefix? "left-acceptance: " <>) lines)))
+         (component-accepts (and lines
+                                 (find (cute string-prefix? "left-acceptance: " <>) lines)))
          (component-accepts (and component-accepts (substring component-accepts (+ 2 (string-contains component-accepts ": ")))))
 
          (component-accepts (and component-accepts (hide-internal-labels component-accepts)))
@@ -689,7 +679,8 @@ init for MODEL unless INIT."
 
          (component-accepts (and component-accepts (string-split component-accepts #\newline)))
          (component-accepts (and component-accepts (sort component-accepts string<?)))
-         (interface-accepts (and lines (filter (cut string-prefix? "right-acceptance: " <>) lines)))
+         (interface-accepts (and lines
+                                 (filter (cute string-prefix? "right-acceptance: " <>) lines)))
          (interface-accepts (and (pair? interface-accepts) (car interface-accepts)))
          (interface-accepts (and interface-accepts (substring interface-accepts (+ 2 (string-contains interface-accepts ": ")))))
 
@@ -704,6 +695,27 @@ init for MODEL unless INIT."
       (throw 'programming-error (format #f "status: ~s, trace: ~s\n" status trace)))
     (values trace interface-accepts component-accepts)))
 
+(define (mcrl2:verify-interface-asserts model root)
+  (let* ((model-name (makreel:unticked-dotted-name model))
+         (asserts (verify-pipeline "verify-interface" root model))
+         (asserts (result-split asserts))
+         (nondets (verify-pipeline "verify-interface-nondet" root model))
+         (nondets (result-split nondets))
+         (lts-tags (get-tags asserts))
+         (unreachable (assert-unreachable lts-tags (model-tags model)))
+         (result `(,unreachable
+                   ,@asserts
+                   ,@nondets))
+         (deadlock? (get-trace 'deadlock result)))
+    (define* (report-assert assert #:key skip?)
+      (report assert skip? (get-trace assert result) model))
+    (reduce-or (command-line:get 'all)
+               `(,(cute report-assert 'deadlock)
+                 ,@(if (%no-unreachable?) '()
+                       `(,(cut report-assert 'unreachable #:skip? deadlock?)))
+                 ,(cute report-assert 'livelock)
+                 ,(cute report-assert 'deterministic)))))
+
 (define (mcrl2:verify-component-asserts model root)
   (let* ((model-name (makreel:unticked-dotted-name model))
          (result status (verify-pipeline (component-stage) root model))
@@ -737,11 +749,9 @@ init for MODEL unless INIT."
   (mcrl2:verify-interface-asserts model root))
 
 (define (mcrl2:verify-component root model)
-  (let* ((component model)
-         (interfaces (delete-duplicates (map .type (ast:port* component)) ast:eq?))
-         (verify-models (append (map (lambda (i) (cut mcrl2:verify-interface-asserts i root)) interfaces)
-                                (list (cut mcrl2:verify-component-asserts component root)))))
-    (reduce-or (command-line:get 'all) verify-models)))
+  (let ((component model))
+    (reduce-or (command-line:get 'all)
+               (list (cut mcrl2:verify-component-asserts component root)))))
 
 (define (mcrl2:verify root model-name)
   (let ((model (makreel:get-model root model-name)))
@@ -758,21 +768,23 @@ init for MODEL unless INIT."
   (let ((model (makreel:get-model root model-name)))
     (unmemoized-verify-pipeline out root model #:stdout? #t)))
 
-(define* (verification:verify options root #:key all? model-name)
+(define* (verification:verify options root #:key all? model-name
+                              no-interfaces?)
   (define (model-names-for-verification root)
     (let* ((models (ast:model** root))
-           (components (filter (conjoin (is? <component>) (negate ast:imported?) .behavior) models))
+           (components (filter (conjoin (is? <component>) (negate ast:imported?)
+                                        .behavior) models))
            (component-names (map makreel:unticked-dotted-name components))
            (interfaces (filter (is? <interface>) models))
            (interface-names (map makreel:unticked-dotted-name interfaces))
-           (interface-names (let loop ((components components) (interface-names interface-names))
-                              (if (null? components) interface-names
-                                  (let ((component-interfaces (map (compose makreel:unticked-dotted-name .type) (ast:port* (car components)))))
-                                    (loop (cdr components)
-                                          (filter (negate (cut member <> component-interfaces)) interface-names)))))))
-      (append interface-names component-names)))
-  (let ((model-names (if model-name (list model-name)
-                         (model-names-for-verification root))))
+           (component-interface-names
+            (map (compose makreel:unticked-dotted-name .type)
+                 (append-map ast:port* components)))
+           (interface-names (append interface-names component-interface-names))
+           (interface-names (delete-duplicates interface-names)))
+      (if no-interfaces? component-names
+          (append interface-names component-names))))
+  (let ((model-names (model-names-for-verification root)))
     (let loop ((model-names model-names) (error? #f))
       (if (or (and (not all?) error?) (null? model-names)) (if error? 1 0)
           (let* ((model-name (car model-names))
