@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
 ;;; Copyright © 2019, 2020, 2021, 2022, 2023, 2024 Janneke Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2020, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2020, 2021, 2022, 2023, 2024 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -93,7 +93,6 @@ format."
          (port-trace-prefix
           (if (not (.status (car port-trace-prefix))) port-trace-prefix
               (cdr port-trace-prefix)))
-         (port-start (car port-trace-prefix))
          (port-instance (and=> (find .instance port-trace) .instance))
          (other-port (runtime:other-port port-instance))
          (instance (.container other-port))
@@ -237,7 +236,7 @@ provides port, mark the trace as <fork-error>, otherwise return false."
                   (list (cons pc trace))))))))
 
 (define (run-provides-modeling pc port-instance event)
-  (%debug "run-provides-modeling... ~a\n" event)
+  (%debug (current-source-location) "run-provides-modeling... ~a" event)
   (let* ((ipc (clone pc #:instance port-instance #:statement #f))
          (interface ((compose .type .ast) port-instance))
          (modeling-names (modeling-names interface)))
@@ -247,10 +246,15 @@ provides port, mark the trace as <fork-error>, otherwise return false."
       (append-map (cute run-to-completion ipc <>) modeling-names))))
 
 (define (run-provides-port pc port-instance event)
-  (%debug "run-provides-port... ~a ~a\n" port-instance event)
-  (let* ((ipc (clone pc #:previous #f #:trail '() #:status #f #:statement #f))
+  (%debug (current-source-location) "run-provides-port... ~a ~a" port-instance event)
+  (let* ((ipc (clone pc
+                     #:instance #f
+                     #:previous #f
+                     #:statement #f
+                     #:status #f
+                     #:trail '()))
          (ipc (reset-reply ipc port-instance)))
-    (%debug "  ipc: ~a\n" ipc)
+    (%debug "  ipc: ~a" ipc)
     (parameterize ((%sut port-instance)
                    (%exploring? #t))
       (run-to-completion ipc event))))
@@ -280,10 +284,10 @@ provides port, mark the trace as <fork-error>, otherwise return false."
                          ;; Check prefix only as long as trace is blocked
                          (list-head port-trail (min (length port-trail)
                                                     (length sut-trail)))))
-         (foo (%debug "     port trail : ~s\n" (map cdr port-trail)))
+         (foo (%debug (current-source-location) "     port trail : ~s" (map cdr port-trail)))
          (port-name ((compose .name .ast) port-instance))
          (events (map (compose last (cut string-split <> #\.) cdr) sut-trail))
-         (foo (%debug "      sut trail : ~s\n\n" events)))
+         (foo (%debug (current-source-location) "      sut trail : ~s" events)))
     (or (any non-matching-pair? port-trail sut-trail)
         (let ((port-length (length port-trail))
               (sut-length (length sut-trail)))
@@ -363,7 +367,7 @@ Return a list of traces, possibly marked with <compliance-error>."
               (if (not internal?) (runtime:port-name->instance port-name)
                   (runtime:port instance port))))
 
-        (%debug "check-provides-compliance... ~s: ~a [~a]\n" port-name event port-event)
+        (%debug (current-source-location) "check-provides-compliance... ~s: ~a [~a]" port-name event port-event)
         (let* ((port-traces
                 (if port-event (run-provides-port pc port-instance port-event)
                     (run-provides-modeling pc port-instance event)))
@@ -388,38 +392,86 @@ Return a list of traces, possibly marked with <compliance-error>."
                    (trail (map cdr trail)))
               (member event-name trail)))
 
-          (when (> (dzn:debugity) 0)
-            (%debug "[c] sut-trail:~s\n" (map cdr sut-trail))
-            (%debug "[c] port-traces[~s]:\n" (length port-traces))
-            (parameterize ((%sut port-instance))
-              (display-trails port-traces)))
+          (define (truncate-to-observable trace count)
+            (define observable?
+              (conjoin
+               trace->trail
+               (compose not string-null? cdr trace->trail)
+               (if (is-a? (%sut) <runtime:port>) (const #t)
+                   (compose (cute string-prefix? port-prefix <>)
+                            cdr trace->trail))))
+            (let* ((trace (reverse trace))
+                   (trace
+                    (let loop ((trace trace) (count count))
+                      (let ((i (list-index observable? trace)))
+                        (if (not i) (if (zero? count) '() trace)
+                            (if (zero? count) '()
+                                (let ((prefix trace (split-at trace (1+ i))))
+                                  (append prefix
+                                          (loop trace (1- count))))))))))
+              (reverse trace)))
 
-          (let ((port-traces
-                 non-compliances
-                 (partition (negate first-non-match) port-traces)))
+          (when (> (dzn:debugity) 0)
+            (%debug (current-source-location) "[c] sut-trail:~s" (map cdr sut-trail))
+            (%debug (current-source-location) "[c] port-traces[~s]:" (length port-traces))
+            (for-each (lambda (i trace)
+                        (let* ((trail (parameterize ((%sut port-instance))
+                                        (trace->string-trail trace)))
+                               (pc (car trace))
+                               (state (serialize (get-state pc port-instance))))
+                          (format (current-error-port) "  ~a: ~a\n" i trail)
+                          (format (current-error-port) "     ~a\n" state)))
+                      (iota (length port-traces))
+                      port-traces)
+            (newline (current-error-port)))
+
+          (let* ((all-port-traces port-traces)
+                 (port-traces non-compliances
+                              (partition (negate first-non-match) port-traces))
+                 (sut-trail-length (length sut-trail)))
             (cond
-             ((and (pair? (append port-traces non-compliances))
+             ((and (pair? all-port-traces)
                    (every (compose (disjoin (is-status? <illegal-error>)
                                             (is-status? <implicit-illegal-error>))
                                    car)
                           (append port-traces non-compliances)))
-              (%debug "  exit 0\n")
+              (%%debug (current-source-location) "  exit 0")
               (map (cute zip trigger <> <>) traces
                    (append port-traces non-compliances)))
              ((and (pair? trace)
-                   (.status (car trace))
-                   (not (is-a? (.status (car trace)) <match-error>))
-                   (pair? (append port-traces non-compliances)))
-              (%debug "  exit 1\n")
+                   (and=> (.status (car trace)) (negate (is? <match-error>)))
+                   (pair? all-port-traces))
+              (%%debug (current-source-location) "  exit 1")
               (let* ((statement (.statement (car trace)))
                      (trace (rewrite-trace-head (cut clone <> #:statement #f) trace))
                      (trace (zip trigger trace (car (append port-traces non-compliances))))
                      (trace (rewrite-trace-head (cut clone <> #:statement statement) trace)))
                 (list trace)))
+             ((and (pair? port-traces)
+                   (pair? trace)
+                   (null? sut-trail))
+              (%%debug (current-source-location) "  exit 2-")
+              (list trace))
              ((and (pair? port-traces)
                    (pair? trace))
-              (%debug "  exit 2\n")
-              (let* ((port-pcs (map
+              (%%debug (current-source-location) "  exit 2")
+              (let* ((port-trails (parameterize ((%sut port-instance))
+                                    (map trace->string-trail port-traces)))
+                     (shortest (apply min (map length port-trails)))
+                     (truncate? (and (pair? sut-trail)
+                                     (not (is-a? (car (last sut-trail))
+                                                 <trigger-return>))
+                                     (< sut-trail-length shortest)
+                                     (and trigger
+                                          (not (and=> (.port trigger)
+                                                      ast:requires?)))))
+                     (port-traces (if (not truncate?) port-traces
+                                      (parameterize ((%sut port-instance))
+                                        (map (cute truncate-to-observable
+                                                   <>
+                                                   sut-trail-length)
+                                             port-traces))))
+                     (port-pcs (map
                                 (compose (cut clone pc #:state <>) .state car)
                                 port-traces))
                      (pc (car trace))
@@ -428,34 +480,45 @@ Return a list of traces, possibly marked with <compliance-error>."
                      (tail (cdr trace))
                      (traces (map (cute cons <> tail) pcs)))
                 (map (cute zip trigger <> <>) traces port-traces)))
+             ((and (%compliance-check?)
+                   (null? non-compliances)
+                   (not blocking?)
+                   (null? port-traces)
+                   (pair? sut-trail)
+                   (let* ((pc (car trace))
+                          (status (.status pc)))
+                     (or (is-a? status <illegal-error>)
+                         (is-a? status <implicit-illegal-error>))))
+              (%%debug (current-source-location) "  exit 3-")
+              (list trace))
              ((and (%compliance-check?)
                    (null? non-compliances)
                    (not blocking?)
                    (null? port-traces)
                    (pair? sut-trail))
-              (%debug "  exit 3\n")
+              (%%debug (current-source-location) "  exit 3")
               (let ((status (make <compliance-error>
                               #:message "non-compliance"
                               #:component-acceptance (caar sut-trail)
                               #:port port-instance)))
                 (list (rewrite-trace-head (cut clone <> #:status status) trace))))
              ((null? non-compliances)
-              (%debug "  exit 4\n")
+              (%%debug (current-source-location) "  exit 4")
               (if (null? trace) '()
                   (list trace)))
              ((and (not port-event)
                    (null? sut-trail)
                    (pair? trace))
-              (%debug "  exit 5\n")
+              (%%debug (current-source-location) "  exit 5")
               (list trace))
              ((let* ((port-instance (any .instance trace))
                      (container (and=> port-instance .container)))
                 (is-a? container <runtime:foreign>))
-              (%debug "  exit 6\n")
+              (%%debug (current-source-location) "  exit 6")
               (map (cute zip trigger trace <>)
                    (append port-traces non-compliances)))
              ((%compliance-check?)
-              (%debug "  exit 7\n")
+              (%%debug (current-source-location) "  exit 7")
               (let* ((port-acceptances (map first-non-match non-compliances))
                      (port-acceptances (delete-duplicates port-acceptances
                                                           port-acceptance-equal?))
@@ -482,6 +545,19 @@ Return a list of traces, possibly marked with <compliance-error>."
                            (not (any (cute event-on-trail? (.event.name trigger) <>)
                                      non-compliances))
                            trigger))
+                     (port-trails (parameterize ((%sut port-instance))
+                                    (map trace->string-trail non-compliances)))
+                     (shortest (apply min (map length port-trails)))
+                     (truncate? (>= (length sut-trail) shortest))
+                     (trace (if (not truncate?) trace
+                                (truncate-to-observable trace shortest)))
+                     (acceptance? (conjoin
+                                   .statement
+                                   (compose
+                                    (cute ast:equal? <> component-acceptance)
+                                    .statement)))
+                     (trace (if (null? sut-trail) trace
+                                (drop-while (negate acceptance?) trace)))
                      (pc (clone pc
                                 #:previous #f
                                 #:status (make <compliance-error>
@@ -491,11 +567,10 @@ Return a list of traces, possibly marked with <compliance-error>."
                                            #:port-acceptance port-acceptances
                                            #:trigger compliance-trigger))))
                 (if (null? trace) (list (cons pc (car non-compliances)))
-                    (let* ((tail (cdr trace))
-                           (trace (cons pc tail)))
+                    (let ((trace (cons pc trace)))
                       (list (zip trigger trace (car non-compliances)))))))
              (else
-              (%debug "  exit 8\n")
+              (%%debug (current-source-location) "  exit 8")
               (let* ((port-trace (car non-compliances))
                      (port-pc (last port-trace))
                      (port-instance (.instance port-pc))
@@ -503,11 +578,7 @@ Return a list of traces, possibly marked with <compliance-error>."
                       (rewrite-trace-head
                        (cut update-state <> port-instance port-pc)
                        port-trace))
-                     (trace (zip trigger trace (car non-compliances)))
-                     (trace
-                      (rewrite-trace-head
-                       (cut clone <> #:skip-compliance? #t)
-                       trace)))
+                     (trace (zip trigger trace (car non-compliances))))
                 (list trace))))))))
 
     (define (check-provides-fork-and-zip port trace)
@@ -521,12 +592,14 @@ Return a list of traces, possibly marked with <compliance-error>."
                  (fold check-compliance traces ports))))))
 
     (if port (or (and (%compliance-check?)
+                      (not (%exploring?))
                       (> (length (ast:provides-port* component)) 1)
                       (is-a? (%sut) <runtime:component>)
                       (check-provides-fork-and-zip port trace))
                  (check-compliance port (list trace)))
         (let ((ports (ast:provides-port* component)))
           (or (and (%compliance-check?)
+                   (not (%exploring?))
                    (> (length ports) 1)
                    (is-a? (%sut) <runtime:component>)
                    (or (check-provides-fork-and-zip #f trace)
@@ -541,8 +614,11 @@ on TRACE."
 
   (let* ((event (and=> trigger trigger->string))
          (orig-pc pc)
-         (pc (reset-replies pc))
-         (pc (clone pc #:instance #f))
+         (pc (if trigger (rtc-pc pc)
+                 ;; defer
+                 (let* ((pc (last trace))
+                        (pc (reset-replies pc)))
+                   (clone pc #:instance #f))))
          (internal? (and (is-a? (%sut) <runtime:system>)
                          (not (eq? instance (%sut)))))
          (sut-trail (if (not internal?) (trace->trail trace)
@@ -584,7 +660,7 @@ on TRACE."
                                 trace)))
           (reverse trace)))
 
-      (%debug "update-port-state... ~s: ~a [~a]\n" port-name event port-event)
+      (%debug (current-source-location) "update-port-state... ~s: ~a [~a]" port-name event port-event)
       (let* ((port-traces
               (if port-event (run-provides-port pc port-instance port-event)
                   (run-provides-modeling pc port-instance event)))
@@ -611,8 +687,8 @@ on TRACE."
                                       #:provides-trigger? provides-trigger?))
 
         (when (> (dzn:debugity) 0)
-          (%debug "[u] sut-trail:~s\n" (map cdr sut-trail))
-          (%debug "[u] port-traces[~s]:\n" (length port-traces))
+          (%debug (current-source-location) "[u] sut-trail:~s" (map cdr sut-trail))
+          (%debug (current-source-location) "[u] port-traces[~s]:" (length port-traces))
           (parameterize ((%sut port-instance))
             (display-trails port-traces)))
 
@@ -621,7 +697,7 @@ on TRACE."
                (partition (negate first-non-match) port-traces)))
           (cond
            ((pair? port-traces)
-            (%debug "  [u] exit 0\n")
+            (%debug (current-source-location) "  [u] exit 0")
             (let* ((port-trace (car port-traces))
                    (port-pc (car port-trace))
                    (port-pc (clone pc #:state (.state port-pc)))
@@ -630,7 +706,7 @@ on TRACE."
                    (pc (clone pc #:instance instance)))
               pc))
            (else
-            (%debug "  [u] exit 1\n")
+            (%debug (current-source-location) "  [u] exit 1")
             orig-pc)))))))
 
 (define-method (check-provides-compliance (pc <program-counter>) event trace)
@@ -646,11 +722,16 @@ released by a requires event, also rerun check-provides-compliance for
 the full trace, i.e., starting from the initial blocking provides
 event.  This ensures proper of zipping the port trace, including the
 port return."
-  (let* ((skip? (blocked-on-boundary? (car trace) event))
+  (let* ((blocked (.blocked pc))
+         (defer-q (.defer pc))
+         (skip? (or (and (pair? blocked)
+                         (requires-trigger? event))
+                    (and (null? defer-q)
+                         (return-trigger? event))
+                    (blocked-on-boundary? pc event)))
          (traces (if skip? (list trace)
                      (check-provides-compliance pc event trace)))
          (pc (car trace))
-         (blocked (.blocked pc))
          (collateral (.collateral pc))
          (compliance-for-blocking?
           (or (find blocked-on-boundary? trace)
@@ -681,6 +762,7 @@ port return."
             (if (null? pcs) traces
                 (let* ((trail (trace->trail (car pcs)))
                        (event (match trail ((ast . event) event) (_ event)))
+                       (event (format #f "~a" event))
                        (cpc (last trace))
                        (cpc (reset-replies cpc))
                        (cpc (clone cpc #:instance #f))
@@ -689,7 +771,6 @@ port return."
                                            ast:provides?)
                                     (equal? (trigger->string rtc-block-trigger)
                                             event))
-                               (symbol? event)
                                (not (.port (string->trigger event))))))
                   (if skip? (loop traces (cdr pcs))
                       (loop
@@ -740,11 +821,12 @@ update TRACES."
                                     (not (runtime:boundary-port? r:other-port))
                                     (null? (.blocked pc)))))
 
-    (%debug "internal-check-compliance... ~s: ~a [~a]\n"
+    (%debug (current-source-location) "internal-check-compliance... ~s: ~a [~a]"
             (trace-name r:port) (trigger->string trigger) internal-compliance?)
     (if (not internal-compliance?) traces
         (let* ((sut-trace (list-head trace (1+ start-index)))
-               (cpc (reset-replies pc))
+               (cpc (last sut-trace))
+               (cpc (reset-replies cpc))
                (cpc (clone cpc #:instance #f))
                (event (trigger->string trigger))
                (compliance-traces
