@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2019, 2020, 2021, 2022, 2023, 2025 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2020, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
 ;;;
@@ -177,10 +177,10 @@ valid PCs that are executing an imperative statement."
 <trigger-return> on a blocking requires port, unless already blocked on
 this statement.  Otherwise, return #f.
 
-Blocking and resettiing the STATEMENT on the TRACE makes RTC?  true,
-which creates an opening to accept new modeling events.  Continuation of
-the <action> or <trigger-return> happens when revisiting this method
-with a TRACE that has BLOCKED still set to this port."
+Blocking and resetting the #:statement on TRACE makes RTC? true, which
+creates an opening to accept new modeling events.  Continuation of the
+<action> or <trigger-return> happens when revisiting this method with a
+TRACE that has BLOCKED still set to this port."
 
   (define (modeling-traces pc instance)
     (let* ((port (.ast instance))
@@ -297,7 +297,7 @@ prefix."
             (%livelock-threshold (* 2 (%livelock-threshold)))
             #f)
            ((pc tail ...)
-            (%debug "  ~s ~s <livelock>\n"
+            (%debug (current-source-location) "  ~s ~s <livelock>"
                     ((compose name .instance) pc)
                     (and=> (.trigger pc) trigger->string))
             (let ((index (and=> (list-index (cute pc-equal? <> pc)
@@ -314,9 +314,8 @@ program-counters produced by taking a step."
 
   (define* ((mark-pc input orig-pc) pc)
     (let ((statement (.statement orig-pc)))
-      (%debug "match fail, ast ~s ~a, input ~s\n" (name statement)
-              (and (is-a? statement  <action>)
-                   (and=> (trace->trail orig-pc) cdr))
+      (%debug (current-source-location) "match fail, ast ~s ~a, input ~s" (name statement)
+              (and=> (trace->trail orig-pc) cdr)
               input)
       (cond ((.status pc)
              pc)
@@ -339,6 +338,7 @@ program-counters produced by taking a step."
 
   (let loop ((trace trace))
     (let ((pc (car trace)))
+      (%debug pc)
       (cond ((.status pc)
              (list trace))
             ((livelock? trace)
@@ -353,6 +353,7 @@ program-counters produced by taking a step."
                     (instance (.instance pc))
                     (update-state?
                      (and (is-a? instance <runtime:component>)
+                          (not (is-a? (%sut) <runtime:port>))
                           (is-a? o <action>)
                           (ast:out? o)))
                     (pc (if (not update-state?) pc
@@ -487,16 +488,26 @@ PC until RTC?."
       (loop (append-map extend-trace traces))))
 
   (define (interface-non-deterministic? traces)
-    (let ((traces (filter (compose (is? <action>) .ast .status car) traces)))
+    (let loop ((traces traces))
       (match traces
         (((pc0 tail0 ...) (pcs tails ...) ...)
          (define (state-equal? pc0 pc1)
            (let ((variables0 (get-variables pc0 (%sut)))
                  (variables1 (get-variables pc1(%sut))))
              (ast:equal? variables0 variables1)))
-         (find (negate (cute state-equal? pc <>)) pcs))
+         (let ((trail0 (trace->string-trail tail0))
+               (trails (filter-map (lambda (pc tail)
+                                     (and (not (state-equal? pc0 pc))
+                                          (trace->string-trail tail)))
+                                   pcs tails)))
+           (or (member trail0 trails)
+               (loop (map cons pcs tails)))))
         (() #f))))
 
+  (define (interface-postponed-non-deterministic? traces)
+    (let ((traces (filter (compose (is? <action>) .ast .status car) traces)))
+      (interface-non-deterministic? traces)))
+
   (define (interface-mark-determinism-error trace)
     (match trace
       ((pc-postponed pc trace ...)
@@ -560,12 +571,17 @@ PC until RTC?."
             (reset-posponed-match traces))
            ((and (is-a? (%sut) <runtime:port>)
                  (%strict?)
-                 (interface-non-deterministic? traces))
+                 (interface-postponed-non-deterministic? traces))
             (map interface-mark-determinism-error traces))
            ((and (not (%exploring?)) (interactive?))
             (choose-postponed-match traces))
            (else
             (mark-end-of-trail traces)))))
+       ((and (is-a? (%sut) <runtime:port>)
+             (not (%exploring?))
+             (%strict?)
+             (interface-non-deterministic? traces))
+        (map interface-mark-determinism-error traces))
        (else
         (loop (append-map extend-trace traces))))))
 
@@ -596,11 +612,12 @@ with EVENT as first step, until RTC?."
                                      (else (trigger->string event))))
                  (sut (%sut))
                  (root (ast:parent (.ast sut) <root>))
-                 (key (string-append
-                       (number->string (.id root))
-                       (parameterize ((%sut #f)) (runtime:dotted-name sut))
-                       (pc->string pc)
-                       event-string)))
+                 (key (parameterize ((%sut #f))
+                        (string-append
+                         (number->string (.id root))
+                         (runtime:dotted-name sut)
+                         (pc->string pc)
+                         event-string))))
             (gc-buffer key)
             (or (hash-ref cache key)
                 (let ((result (run-to-completion-unmemoized pc event)))
@@ -641,13 +658,14 @@ until RTC?."
     (run-to-completion-unmemoized pc)))
 
 (define-method (run-flush (pc <program-counter>) (instance <runtime:component>))
-  (let ((pc (clone pc #:instance instance)))
-    (run-flush pc)))
+  (if (.status pc) (list (list pc))
+      (let ((pc (clone pc #:instance instance)))
+        (run-flush pc))))
 
 (define-method (run-silent (pc <program-counter>) (port <runtime:port>))
   (define (update-state pc port-pc)
     (set-state pc (get-state port-pc port)))
-  (%debug "run-silent... ~s\n" (name port))
+  (%debug (current-source-location) "run-silent... ~s" (name port))
   (let ((modeling-names (modeling-names port)))
     (if (null? modeling-names) '()
         (let* ((ipc (clone pc #:trigger #f #:previous #f #:instance #f #:trail '() #:statement #f))
@@ -677,7 +695,7 @@ until RTC?."
   (define (update-state pc port-pc)
     (let ((pc (set-state pc (get-state port-pc port))))
       (clone pc #:external-q (.external-q port-pc) #:status (.status port-pc))))
-  (%debug "run-external-modeling... ~s\n" (name port))
+  (%debug (current-source-location) "run-external-modeling... ~s" (name port))
   (let* ((r:other-port (runtime:other-port port))
          (external? (and (ast:requires? r:other-port)
                          (ast:external? r:other-port))))
@@ -745,7 +763,10 @@ until RTC?."
              (modeling-names (if modeling? (list event) (modeling-names)))
              (traces (append-map (cute run-to-completion pc <>) modeling-names))
              (traces (parameterize ((%modeling? modeling?))
-                       (filter event-executed? traces))))
+                       (filter (disjoin
+                                event-executed?
+                                (compose (is-status? <determinism-error>) car))
+                               traces))))
         traces)))))
 
 (define-method (run-requires-flush (pc <program-counter>) event)
@@ -781,7 +802,7 @@ until RTC?."
                       (pc trigger (dequeue-external pc port-instance)))
                  (and trigger
                       (equal? (trigger->string trigger) event)))))))
-  (%debug "run-requires... ~s\n" event)
+  (%debug (current-source-location) "run-requires... ~s" event)
   (let* ((component ((compose .type .ast) (%sut)))
          (trigger (string->trigger event))
          (trigger (clone trigger #:parent component))
@@ -800,7 +821,7 @@ until RTC?."
         (append-map (cute run-requires-flush <> event) traces))))
 
 (define-method (run-defer-event (pc <program-counter>) event)
-  (%debug "run-defer-event ~a pc: ~s\n" event pc)
+  (%debug (current-source-location) "run-defer-event ~a pc: ~s" event pc)
   (let* ((pc (prune-defer pc))
          (defer (.defer pc))
          (trail (.trail pc))
@@ -886,7 +907,7 @@ until RTC?."
       (list trace))))
 
 (define-method (run-external (pc <program-counter>) event)
-  (%debug "run-external ~a pc: ~s\n" event pc)
+  (%debug (current-source-location) "run-external ~a pc: ~s" event pc)
   (let ((queues (.external-q pc)))
     (if (null? queues) '()
         (match (external-trigger-in-q? pc event)
@@ -899,7 +920,7 @@ until RTC?."
                   (error (make <match-error> #:ast ast #:input event
                                #:message "match"))
                   (pc (clone pc #:status error)))
-             (%debug "<match> ~a pc: ~s\n" event pc)
+             (%debug (current-source-location) "<match> ~a pc: ~s" event pc)
              (list (list pc) )))))))
 
 (define-method (run-to-completion* (pc <program-counter>) event)
@@ -909,7 +930,7 @@ until RTC?."
       (list (list (clone pc #:status illegal)))))
   (define (modeling? trace)
     (and=> (any .trigger (reverse trace)) ast:modeling?))
-  (%debug "run-to-completion*: ~a\n" event)
+  (%debug (current-source-location) "run-to-completion*: ~a" event)
   (cond
    ((is-a? (%sut) <runtime:port>)
     (run-interface pc event))
@@ -971,7 +992,7 @@ until RTC?."
             (append-map run-flush traces))))
 
 (define-method (run-to-completion*-context-switch (pc <program-counter>) event)
-  (%debug "run-to-completion*-context-switch: ~a\n" event)
+  (%debug (current-source-location) "run-to-completion*-context-switch: ~a" event)
   (let* ((orig-pc pc)
          (blocked-on-action? (blocked-on-action? pc event))
          (pc (if (and (blocked-on-boundary? pc event)
