@@ -34,15 +34,15 @@
   #:use-module (gnu packages bash)
   #:use-module (gnu packages boost)
   #:use-module (gnu packages guile)
-  #:use-module (gnu packages maths)
   #:use-module (gnu packages pkg-config)
   #:use-module (gnu packages pth)
+  #:use-module (pack mcrl2)
   #:use-module (pack scmackerel))
 
 (define-public dezyne
   (package
     (name "dezyne")
-    (version #!dezyne!# "2.18.2")
+    (version #!dezyne!# "2.18.3")
     (source
      (origin
        (method url-fetch)
@@ -51,7 +51,7 @@
                            "dezyne.org/download/dezyne/"
                            name "-" version ".tar.gz"))
        (sha256
-        (base32 #!dezyne!# "0cya5p7y546ldiycz5clv0r8xhxf8zp8iim50y20vhsfgxvh96is"))))
+        (base32 #!dezyne!# "1c4bi3gpl2fi6pk8z9gmrspg2ad7flkgqjs18bnczswii47yg2s8"))))
     (build-system gnu-build-system)
     (native-inputs (list guile-3.0 pkg-config))
     (inputs (list bash-minimal
@@ -59,14 +59,15 @@
                   guile-3.0
                   guile-json-4
                   guile-readline
-                  mcrl2-minimal
+                  (libc-utf8-locales-for-target)
+                  mcrl2-next-minimal
                   pth
                   scmackerel
                   sed))
     (arguments
      (list
       #:modules `((ice-9 popen)
-                  ,@%gnu-build-system-modules)
+                  ,@%default-gnu-imported-modules)
       #:phases
       #~(modify-phases %standard-phases
           (add-before 'configure 'setenv
@@ -83,7 +84,7 @@
               (let* ((bash #$(this-package-input "bash-minimal"))
                      (guile #$(this-package-input "guile-3.0"))
                      (json #$(this-package-input "guile-json"))
-                     (mcrl2 #$(this-package-input "mcrl2-minimal"))
+                     (mcrl2 #$(this-package-input "mcrl2-next-minimal"))
                      (readline #$(this-package-input "guile-readline"))
                      (scmackerel #$(this-package-input "scmackerel"))
                      (sed #$(this-package-input "sed"))
