@@ -45,18 +45,17 @@ class main
     dzn.Locator locator = new dzn.Locator();
     dzn.Runtime runtime = new dzn.Runtime();
     locator.set (runtime);
-    using(blocking_multiple_provides3 sut = new blocking_multiple_provides3 (locator))
+    using (blocking_multiple_provides3 sut
+      = new blocking_multiple_provides3 (locator, "sut"))
     {
-      sut.dzn_meta.name = "sut";
-
-      sut.r.inport.hello = () =>
+      sut.r.in_port.hello = () =>
       {
         System.Console.Error.WriteLine("sut.mp.r.hello -> <external>.r.hello");
-        sut.mp.r.outport.world();
+        sut.r.out_port.world();
         System.Console.Error.WriteLine("sut.mp.r.return -> <external>.r.return");
       };
-      sut.left.inport.hello ();
-      sut.right.inport.hello ();
+      sut.left.in_port.hello ();
+      sut.right.in_port.hello ();
     }
   }
 }
