@@ -1,6 +1,6 @@
 // Dezyne --- Dezyne command line tools
 //
-// Copyright © 2016, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+// Copyright © 2016, 2021, 2024 Janneke Nieuwenhuizen <janneke@gnu.org>
 // Copyright © 2016 Henk Katerberg <hank@mudball.nl>
 // Copyright © 2022 Rutger van Beusekom <rutger@dezyne.org>
 //
@@ -30,28 +30,24 @@
 
 #include <iostream>
 
-void f()
-{
-  std::clog << "sut.m.t.f -> <external>.t.f" << std::endl;
-}
-
-int main()
+int
+main ()
 {
   std::string str;
-  while(std::cin >> str);
+  while (std::cin >> str);
 
-  dzn::locator l;
-  dzn::runtime rt;
-  l.set(rt);
+  dzn::locator locator;
+  dzn::runtime runtime;
+  locator.set (runtime);
 
-  hello_injected sut(l);
+  hello_injected sut (locator);
   sut.dzn_meta.name = "sut";
-  sut.t.meta.require.name = "t";
-  sut.t.meta.require.component = 0;
-  sut.t.out.f = f;
+  sut.h.dzn_meta.require.name = "h";
+  sut.h.dzn_meta.require.component = 0;
+  sut.h.out.world = [] () {};
 
-  sut.check_bindings();
-  sut.dump_tree();
+  dzn::check_bindings (sut);
+  dzn::dump_tree (sut);
 
-  sut.t.in.e();
+  sut.h.in.hello ();
 }
