@@ -1,8 +1,8 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
 ;;; Copyright © 2019 Timothy Sample <samplet@ngyro.com>
-;;; Copyright © 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
+;;; Copyright © 2019, 2020, 2021, 2022, 2023, 2024 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2021, 2025 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2021, 2022, 2023, 2024 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2023 Karol Kobiela <karol.kobiela@verum.com>
 ;;;
@@ -41,6 +41,7 @@
   #:use-module (dzn shell-util)
   #:use-module (dzn trace)
   #:export (%default-stages
+            get-meta-option
             run-baseline
             run-test))
 
@@ -93,10 +94,11 @@ output, and standard error as three values."
               ((ex-input . ex-output) (pipe)))
     (format #t "~a\n" (string-join
                        (map (lambda (s)
-                              (if (or (string-index s #\space)
-                                      (string-index s #\newline))
-                                  (format #f "~s" s)
-                                  s))
+                              (let ((quote? (or (string-index s #\space)
+                                                (string-index s #\newline)
+                                                (string-index s #\<))))
+                                (if (not quote?) s
+                                    (format #f "~s" s))))
                             command)))
     (match (primitive-fork)
       (0 (catch #t
@@ -121,7 +123,7 @@ output, and standard error as three values."
            (close-port stderr-output)
            (close-port ex-output)
            (display input stdin-output)
-           (close stdin-output)
+           (false-if-exception (close-port stdin-output))
            (match (get-strings-all stdout-input stderr-input ex-input)
              ((stdout stderr "")
               (match-let (((pid . status) (waitpid pid)))
@@ -169,11 +171,19 @@ output, and standard error as three values."
                  stages)
          (format #t "skip\n"))))
 
-(define (get-meta-flag file-name flag)
+(define (get-meta-entry file-name option)
   (let ((alist (get-meta file-name)))
     (and alist
-         (and=> (assq-ref alist flag)
-                (cute equal? <> '(#t))))))
+         (assq option alist))))
+
+(define (get-meta-option file-name option)
+  (let ((alist (get-meta file-name)))
+    (and alist
+         (and=> (assq-ref alist option) car))))
+
+(define (get-meta-flag file-name flag)
+  (and=> (get-meta-option file-name flag)
+         (cute equal? <> #t)))
 
 (define (fall-back? file-name)
   (get-meta-flag file-name 'fall-back))
@@ -185,19 +195,13 @@ output, and standard error as three values."
   (get-meta-flag file-name 'no-unreachable))
 
 (define (queue-size file-name)
-  (let ((alist (get-meta file-name)))
-    (and alist
-         (and=> (assq-ref alist 'queue-size) car))))
+  (get-meta-option file-name 'queue-size))
 
 (define (queue-size-defer file-name)
-  (let ((alist (get-meta file-name)))
-    (and alist
-         (and=> (assq-ref alist 'queue-size-defer) car))))
+  (get-meta-option file-name 'queue-size-defer))
 
 (define (queue-size-external file-name)
-  (let ((alist (get-meta file-name)))
-    (and alist
-         (and=> (assq-ref alist 'queue-size-external) car))))
+  (get-meta-option file-name 'queue-size-external))
 
 (define (thread-pool? file-name)
   (get-meta-flag file-name 'thead-pool))
@@ -206,42 +210,30 @@ output, and standard error as three values."
   (get-meta-flag file-name 'tss))
 
 (define (code-options file-name)
-  (let ((alist (get-meta file-name)))
-    (or (and alist
-             (and=> (assq-ref alist 'code-options) car))
-        '())))
+  (or (get-meta-option file-name 'code-options)
+      '()))
 
 (define (component? file-name)
-  (or (let ((alist (get-meta file-name)))
-        (and alist
-             (and=> (assq-ref alist 'component) car)))
-      (model? file-name)))
+  (or (get-meta-option file-name 'component)
+      (and (not (component-unset? file-name))
+           (model? file-name))))
 
 (define (component-unset? file-name)
-  (let ((alist (get-meta file-name)))
-    (and alist
-         (and=> (assq 'component alist)
-                (cute equal? <> '(component #f))))))
+  (and=> (get-meta-entry file-name 'component)
+         (cute equal? <> '(component #f))))
 
 (define (model? file-name)
-  (let ((alist (get-meta file-name)))
-    (and alist
-         (and=> (assq-ref alist 'model) car))))
+  (get-meta-option file-name 'model))
 
 (define (trace-format file-name)
-  (let ((alist (get-meta file-name)))
-    (and alist
-         (and=> (assq-ref alist 'trace-format) car))))
+  (get-meta-option file-name 'trace-format))
 
 (define (non-strict? file-name)
-  (let ((alist (get-meta file-name)))
-    (and alist
-         (and=> (assq-ref alist 'non-strict?) car))))
+  (get-meta-option file-name 'non-strict?))
 
 (define (simulate-flags file-name)
   (let ((alist (get-meta file-name)))
-    (or (and alist
-             (and=> (assq-ref alist 'simulate-flags) car))
+    (or (get-meta-option file-name 'simulate-flags)
         '())))
 
 
@@ -259,7 +251,7 @@ output, and standard error as three values."
   (or (file-exists? (string-append file-name "/baseline/code.out"))
       (file-exists? (string-append file-name "/baseline/code.err"))))
 
-(define (features file-name)
+(define (file-name->features file-name)
   (define (feature? feature)
     (and (string-contains file-name feature) feature))
   (filter identity
@@ -281,6 +273,13 @@ output, and standard error as three values."
                     "shell"))
            (feature? "space"))))
 
+(define (features file-name)
+  (let* ((features (or (get-meta-option file-name 'features)
+                       '()))
+         (features (append features
+                           (file-name->features file-name))))
+    (delete-duplicates features)))
+
 (define (features-missing file-name language)
   (fold (lambda (feature missing)
           (if (member language (assoc-ref %feature-alist feature)) missing
@@ -310,6 +309,18 @@ output, and standard error as three values."
          (events (filter (negate (cute string-prefix? "(state " <>)) lines)))
     (string-join events "\n")))
 
+(define (trace->base+number name)
+  (let ((m (string-match "^(([^0-9]|[0-9][^0-9])*)([0-9]+)$" name)))
+    (if (not m) (values name #f)
+        (values (match:substring m 1)
+                (string->number (match:substring m 3))))))
+
+(define (trace< a b)
+  (let ((a-base a-number (trace->base+number a))
+        (b-base b-number (trace->base+number b)))
+    (if (and a-number b-number (equal? a-base b-base)) (< a-number b-number)
+        (string< a b))))
+
 
 ;;;
 ;;; Stage runners.
@@ -325,7 +336,7 @@ output, and standard error as three values."
          (includes (filter directory-exists? includes))
          (includes (append-map (cute list "-I" <>) includes))
          (fall-back? (fall-back? file-name))
-         (model (or (component? file-name) (file->model base-name)))
+         (model (component? file-name))
          (no-unreachable? (no-unreachable? file-name))
          (queue-size (queue-size file-name))
          (queue-size-defer (queue-size-defer file-name))
@@ -341,13 +352,14 @@ output, and standard error as three values."
                 ,@(if (not no-unreachable?) '()
                       `("--no-unreachable"))
                 ,@(if (not queue-size) '()
-                      `("-q" ,(number->string queue-size)))
+                      `("--queue-size" ,(number->string queue-size)))
                 ,@(if (not queue-size-defer) '()
                       `("--queue-size-defer" ,(number->string queue-size-defer)))
                 ,@(if (not queue-size-external) '()
                       `("--queue-size-external"
                         ,(number->string queue-size-external)))
-                ,@(if (component-unset? file-name) '() `("-m" ,model))
+                ,@(if (not model) '()
+                      `("--model" ,model))
                 ,dzn-name))))
     (or (skip? file-name "verify")
         (run-baseline file-name command
@@ -371,10 +383,11 @@ output, and standard error as three values."
               (command
                `("dzn" "code"
                  ,@includes
-                 "-l" ,language
-                 "-o" ,out-lang
-                 "-m" ,model
-                 ,@(if (thread-safe-shell? file-name) `("-s" ,base-name) '())
+                 "--language" ,language
+                 "--output" ,out-lang
+                 "--model" ,model
+                 ,@(if (thread-safe-shell? file-name) `("--shell" ,base-name)
+                       '())
                  ,@(code-options file-name)
                  ,dzn-name)))
          (or (error-model? file-name)
@@ -397,7 +410,10 @@ output, and standard error as three values."
                  (lambda (dzn-file)
                    (receive (status stdout stderr)
                        (observe
-                        `("dzn" "code" ,@includes "-l" ,language "-o" ,out-lang
+                        `("dzn" "code"
+                          ,@includes
+                          "--language" ,language
+                          "--output" ,out-lang
                           ,@(code-options file-name)
                           ,dzn-file) input)
                      (zero? status)))
@@ -405,7 +421,8 @@ output, and standard error as three values."
        (or (code-error? file-name)
            (run-build file-name language))
        (or (code-error? file-name)
-           (let ((traces (find-files file-name ".*trace.*$")))
+           (let* ((traces (find-files file-name ".*trace.*$"))
+                  (traces (sort traces trace<)))
              (and-map (cute run-execute file-name language <>) traces)))))
 
 (define (run-traces file-name)
@@ -434,12 +451,12 @@ output, and standard error as three values."
             (observe
              `("dzn" "traces"
                ,@includes
-               "-m" ,model
-               "-o" ,out-lang
+               "--model" ,model
+               "--output" ,out-lang
                ,@(if (null? handwritten-traces) '("--traces") '())
                ,@(if (flush? file-name) '("--flush") '())
                ,@(if (not queue-size) '()
-                     `("-q" ,(number->string queue-size)))
+                     `("--queue-size" ,(number->string queue-size)))
                ,@(if (not queue-size-defer) '()
                      `("--queue-size-defer" ,(number->string queue-size-defer)))
                ,@(if (not queue-size-external) '()
@@ -477,7 +494,7 @@ are weak-bisim equivalent"
              (pipeline->string
               `(("dzn" "parse"
                  ,@includes
-                 "-m" ,model
+                 "--model" ,model
                  "--list-models"
                  ,dzn-name)))))
         (and (zero? status)
@@ -493,8 +510,10 @@ are weak-bisim equivalent"
                 `("dzn" "traces"
                   "--lts"
                   ,@includes
-                  "-m" ,model
-                  ,@(if queue-size `("-q" ,(number->string queue-size)) '())
+                  "--model" ,model
+                  ,@(if (not queue-size) '()
+                        `("--queue-size"
+                          ,(number->string queue-size)))
                   ,dzn-name)
                 input))
               (status-unconstrained
@@ -503,10 +522,12 @@ are weak-bisim equivalent"
                (observe
                 `("dzn" "traces"
                   "--lts"
-                  "-C"
+                  "--no-constraint"
                   ,@includes
-                  "-m" ,model
-                  ,@(if queue-size `("-q" ,(number->string queue-size)) '())
+                  "--model" ,model
+                  ,@(if (not queue-size) '()
+                        `("--queue-size"
+                          ,(number->string queue-size)))
                   ,dzn-name)
                 input)))
           (with-output-to-file constrained
@@ -555,7 +576,7 @@ are weak-bisim equivalent"
              (pipeline->string
               `(("dzn" "parse"
                  ,@includes
-                 "-m" ,model
+                 "--model" ,model
                  "--list-models"
                  ,dzn-name)))))
         (and (zero? status)
@@ -571,8 +592,9 @@ are weak-bisim equivalent"
                 `("dzn" "traces"
                   "--lts"
                   ,@includes
-                  "-m" ,model
-                  ,@(if queue-size `("-q" ,(number->string queue-size)) '())
+                  "--model" ,model
+                  ,@(if (not queue-size) '()
+                        `("--queue-size" ,(number->string queue-size)))
                   ,dzn-name)
                 input))
               (status-no-compliance
@@ -581,10 +603,11 @@ are weak-bisim equivalent"
                (observe
                 `("dzn" "traces"
                   "--lts"
-                  "-D"
+                  "--no-non-compliance"
                   ,@includes
-                  "-m" ,model
-                  ,@(if queue-size `("-q" ,(number->string queue-size)) '())
+                  "--model" ,model
+                  ,@(if (not queue-size) '()
+                        `("--queue-size" ,(number->string queue-size)))
                   ,dzn-name)
                 input)))
           (with-output-to-file constrained
@@ -614,8 +637,12 @@ are weak-bisim equivalent"
          (out (string-append file-name "/out"))
          (out-lang (string-append out "/" language))
          (thread-pool? (thread-pool? file-name))
+         (shadow? (and (string=? language "c++")
+                       (or (string-contains file-name "alpha")
+                           (string-contains file-name "shadow"))))
          (command `("make"
                     "-f" ,(string-append "test/lib/build." language ".make")
+                    ,@(if shadow? '("WARN_FLAGS=-Wall -Wextra -Werror") '())
                     ,(string-append "LANGUAGE=" language)
                     ,(string-append "IN=" file-name)
                     ,(string-append "OUT=" out-lang)
@@ -697,7 +724,7 @@ are weak-bisim equivalent"
         (observe `("dzn" "simulate"
                    ,@includes
                    ,@(if (not queue-size) '()
-                         `("-q" ,(number->string queue-size)))
+                         `("--queue-size" ,(number->string queue-size)))
                    ,@(if (not queue-size-defer) '()
                          `("--queue-size-defer"
                            ,(number->string queue-size-defer)))
@@ -707,7 +734,7 @@ are weak-bisim equivalent"
                    "--format" ,trace-format
                    ,@(if (non-strict? file-name) '() '("--strict"))
                    ,@(simulate-flags file-name)
-                   "-m" ,model
+                   "--model" ,model
                    ,dzn-name)
                  input)
 
@@ -745,7 +772,8 @@ are weak-bisim equivalent"
   (format #t "** stage: simulate\n")
   (or (skip? file-name
              "simulate")
-      (let ((traces (find-files file-name ".*trace.*$")))
+      (let* ((traces (find-files file-name ".*trace.*$"))
+             (traces (sort traces trace<)))
         (and-map (cute run-simulate-trace file-name <>) traces))))
 
 (define (run-lts file-name)
@@ -776,13 +804,13 @@ are weak-bisim equivalent"
              `("dzn" "graph" "--backend=lts"
                ,@includes
                ,@(if (not queue-size) '()
-                     `("-q" ,(number->string queue-size)))
+                     `("--queue-size" ,(number->string queue-size)))
                ,@(if (not queue-size-defer) '()
                      `("--queue-size-defer" ,(number->string queue-size-defer)))
                ,@(if (not queue-size-external) '()
                      `("--queue-size-external"
                        ,(number->string queue-size-external)))
-               "-m" ,model
+               "--model" ,model
                ,dzn-name)
              input)
           (and (zero? status)
@@ -809,21 +837,51 @@ are weak-bisim equivalent"
                             (blocking (delete-duplicates blocking))
                             (modeling (append modeling '("inevitable" "optional")))
                             (taus (append taus flushes blocking))
-                            (taus+modeling (append taus modeling)))
+                            (taus+modeling (append taus modeling))
+                            (makreel-aut (string-append out "/traces/" base-name ".aut"))
+                            (lts-aut (string-append out-lang "/" base-name ".aut")))
                        (with-output-to-file makreel-lts-file
                          (cute display makreel-lts))
                        ;; FIXME: --structured-output with -eweak-trace still writes to disk
-                       (with-directory-excursion out-lang
-                         (receive (status stdout stderr)
-                             (observe
-                              `("ltscompare" "--counter-example" "--structured-output"
-                                "-eweak-trace"
-                                ,(string-append "--tau=" (string-join taus+modeling ","))
-                                ,(string-append "../traces/" base-name ".aut")
-                                ,(string-append base-name ".aut"))
-                              "")
-                           (display stdout)
-                           (zero? (ltscompare-status? status stdout))))))))))))
+                       (receive (status stdout stderr)
+                           (observe
+                            `("ltscompare"
+                              "--counter-example"
+                              "--structured-output"
+                              "-eweak-trace"
+                              ,(string-append "--tau=" (string-join taus+modeling ","))
+                              ,makreel-aut
+                              ,lts-aut)
+                            "")
+                         (display stdout)
+                         (let ((result (zero? (ltscompare-status? status stdout))))
+                           (unless result
+                             (format (current-error-port) "\nYou may try:\n")
+                             (let ((command
+                                    `("ltscompare"
+                                      "--counter-example"
+                                      "--structured-output"
+                                      "-pweak-trace-ac"
+                                      ,(string-append
+                                        "--tau='"
+                                        (string-join taus+modeling ",")
+                                        "'"))))
+                               (format (current-error-port)
+                                       "    ~a\n"
+                                       (string-join
+                                        `(,@command
+                                          ,makreel-aut
+                                          ,lts-aut)))
+                               (format (current-error-port) "and/or:\n")
+                               (format (current-error-port)
+                                       "    ~a\n"
+                                       (string-join
+                                        `(,@command
+                                          ,lts-aut
+                                          ,makreel-aut)))
+                               (format (current-error-port)
+                                       "manually for more information\n")))
+                           result))))))))))
 
 
 ;;;
