@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2020 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
@@ -30,7 +30,7 @@
 
   #:use-module (dzn ast goops)
   #:use-module (dzn ast util)
-  #:use-module (dzn code makreel)
+  #:use-module (dzn code language makreel)
   #:use-module (dzn parse))
 
 (test-begin "makreel")
@@ -43,25 +43,25 @@
 
 (test-assert "tail-call self"
   (let* ((ast (string->ast "interface i {in void e();behavior {void f () {f ();} on e: {}}}"))
-         (ast (makreel:om ast))
+         (ast (makreel:normalize ast))
          (call (car (tree-collect (is? <call>) ast))))
     (.last? call)))
 
 (test-assert "tail-call other"
   (let* ((ast (string->ast "interface i {in void e();behavior {void g () {} void f () {g ();} on e: {}}}"))
-         (ast (makreel:om ast))
+         (ast (makreel:normalize ast))
          (call (car (tree-collect (is? <call>) ast))))
     (.last? call)))
 
 (test-assert "non tail-call"
   (let* ((ast (string->ast "interface i {in void e();behavior {void f () {g (); bool b = true;} void g () {} on e: {}}}"))
-         (ast (makreel:om ast))
+         (ast (makreel:normalize ast))
          (call (car (tree-collect (is? <call>) ast))))
     (not (.last? call))))
 
-(test-assert "non tail-call valued"
+(test-assert "non tail-call typed"
   (let* ((ast (string->ast "interface i {in void e();behavior {void f () {bool b = g ();} bool g () {return true;} on e: {}}}"))
-         (ast (makreel:om ast))
+         (ast (makreel:normalize ast))
          (call (car (tree-collect (is? <call>) ast))))
     (not (.last? call))))
 
